#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::AgentUI::Guts;

use strict;

use Exporter 'import';
BEGIN { our @EXPORT_OK = qw(get_token_name get_token_type is_token_optional get_optional_index); }

use Carp;

use constant {
	OPTIONAL_RE => qr/^\[(.*)\]$/,
	TYPE_RE => qr/(\w+):(\w+)/,
};

sub get_token_name($) {
	my ($token) = @_;

	if($token =~ TYPE_RE) {
		return $2;
	}
	
	if($token =~ OPTIONAL_RE) {
		return $1;
	}

	return $token;
}

sub get_token_type($) {
	my ($token) = @_;

	if($token =~ TYPE_RE) {
		return $1;
	}

	return $token;
}

sub is_token_optional($) {
	my ($token) = @_;

	if($token =~ OPTIONAL_RE) {
		return 1;
	}

	return 0;
}

sub get_optional_index($) {
	my ($command) = @_;

	my $idx;
	my $c = 0;

	foreach my $arg (@{$command->{ARGUMENT_LIST}}) {
		if($arg =~ OPTIONAL_RE) {
			croak "UI command cannot have more than one optional argument" if defined($idx);
			$idx = $c;
		}
		$c++;
	}

	return $idx;
}

1;
