#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::AgentUI::Hint;

use strict;

use Exporter 'import';
BEGIN { our @EXPORT_OK = qw(get_syntax_hint get_syntax_by_name); }

require SrSv::AgentUI;
our %uis;
*uis = \%SrSv::AgentUI::uis;

use SrSv::AgentUI::Guts qw(get_token_name get_token_type is_token_optional);

sub get_syntax_hint($) {
	my ($command) = @_;

	my $syntax = "\002".$command->{SYNONYMS}[0]."\002";

	foreach my $arg (@{$command->{ARGUMENT_LIST}}) {
		my $name = get_token_name($arg);

		$name = "list of $name" if get_token_type($arg) eq 'LIST';
		
		if(is_token_optional($arg)) {
			$syntax .= " [\037$name\037]";
		} else {
			$syntax .= " \037$name\037";
		}
	}

	return $syntax;
}

sub get_syntax_by_name($) {
	my ($name) = @_;

	my ($ui_name, $cmd_name) = ($name =~ /^(.*) (\S+)$/);

	my $command = $uis{$ui_name}->{COMMAND_INDEX}->{uc $cmd_name};

	if($command) {
		return get_syntax_hint($command);
	} else {
		return undef;
	}
}

1;
