#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::Chan::Users;

=head1 NAME

SrSv::Chan::Users - Get a list of users in a channel.

=cut

use strict;

use Exporter 'import';
BEGIN { our @EXPORT = qw(get_chan_users); }

use SrSv::Process::Init;
use SrSv::MySQL '$dbh';
use SrSv::ChanReg::Agent;

our ($get_chan_users);

proc_init {
	$get_chan_users = $dbh->prepare("SELECT user.nick, chanuser.nickid FROM chanuser, user WHERE chanuser.chan=? AND user.id=chanuser.nickid AND chanuser.joined=1");
};

sub get_chan_users($) {
	my ($chan) = @_;

	my $agent = agent($chan);

	$get_chan_users->execute($chan->{CHAN});

	return [map(+{NICK => $_->[0], ID => $_->[1], AGENT => $agent},
		@{$get_chan_users->fetchall_arrayref()})];
}

1;

__END__

=head1 SYNOPSIS

 use SrSv::Chan::Users;
 
 foreach my $user (@{get_chan_users($chan)}) {
 	...
 }

