#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::ChanControl::Clear;

=head1 NAME

SrSv::ChanControl::Clear - Remove all users from a channel.

=cut

use strict;

use Exporter 'import';
BEGIN { our @EXPORT_OK = qw(clear_users chan_kill); }

use SrSv::Conf qw(services);
use SrSv::ChanReg::Agent qw(agent);
use SrSv::User qw(get_user_nick);

use SrSv::Chan::Users;

use SrSv::ChanControl::Enforcer qw(enforcer_join);

sub clear_users($$;$) {
	my ($chan, $reason, $users) = @_;
	$users = get_chan_users($chan) unless defined($users);
	
	my $cn = $chan->{CHAN};
	my $agent = agent($chan);
	
	enforcer_join($chan);
	ircd::setmode($agent, $cn, '+b', '*!*@*');
	ircd::flushmodes();

	foreach my $user (@$users) {
		next if adminserv::is_ircop($user);
		ircd::kick($agent, $cn, get_user_nick($user), $reason);
	}
}

sub chan_kill($$;$) {
	my ($chan, $reason, $users) = @_;
	$users = get_chan_users($chan) unless defined($users);
	
	my $cn = $chan->{CHAN};
	my $agent = agent($chan);
	
	enforcer_join($chan);
	
	foreach my $user (@$users) {
		next if adminserv::is_ircop($user);
		nickserv::kline_user($user, $services_conf{chankilltime}, $reason);
	}
}

1;
