#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::ChanControl::Close;

=head1 NAME

SrSv::ChanControl::Close - Keep users out of certain channels.

=cut

use strict;

use Exporter 'import';
BEGIN { our @EXPORT_OK = qw(); }

use SrSv::Message qw(add_callback);

use SrSv::ChanReg::Agent;
use SrSv::ChanReg::Flags;
use SrSv::ChanReg::Close qw(get_close);

use SrSv::ChanControl::Enforcer qw(%cs_enforcers);
use SrSv::ChanControl::Clear qw(clear_users chan_kill);

add_callback {
	CALL => \&check_closed,
	TRIGGER_COND => {
		CLASS => 'chanserv',
		TYPE => 'check_join',
	}
};

sub check_closed($$) {
	my ($message, $callback) = @_;

	my ($chan, $users) = ($message->{CHAN}, $message->{USERS});
	
	if(cr_flag($chan, (CRF_CLOSE | CRF_DRONE))) {
		$chan->{__ABORT} = 1;
		my $cn = $chan->{CHAN};
		my ($reason, $opnick, $time) = get_close($chan);
		my $cmsg = "$cn is closed: $reason";
		my $preenforce = $cs_enforcers{lc $chan};
		
		if (cr_flag($chan, CRF_CLOSE)) {
			clear_users($chan, $cmsg, $users);
		}
		elsif (cr_flag($chan, CRF_DRONE)) {
			chan_kill($chan, $cmsg, $users);
		}

		unless($preenforce) {
			ircd::settopic(agent($chan), $cn, $opnick, $time, $cmsg);
		}
		
		return;
	}
}

1;
