#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::ChanControl::Enforcer;

=head1 NAME

SrSv::ChanControl::Enforcer - Make an agent stay in a channel for 1 minute.

=cut

use strict;

use Exporter 'import';
BEGIN { our @EXPORT_OK = qw(%cs_enforcers enforcer_join enforcer_part); }

use SrSv::ChanReg::Agent qw(agent);
use SrSv::ChanControl::ModeLock qw(do_modelock_quick);
use SrSv::Timer qw(add_timer);

use SrSv::Shared qw(%cs_enforcers);

sub enforcer_join($) {
	my ($chan) = @_;
	my $cn = $chan->{CHAN};
	my $bot = agent($chan);

	return if $cs_enforcers{lc $cn};
	$cs_enforcers{lc $cn} = lc $bot;

	botserv::bot_join($chan);
	do_modelock_quick($chan);
	
	add_timer("CSEnforce $bot $cn", 60, __PACKAGE__, 'enforcer_part');
}

sub enforcer_part($) {
	my ($cookie) = @_;
	my ($junk, $bot, $cn) = split(/ /, $cookie);

	return unless $cs_enforcers{lc $cn};
	undef($cs_enforcers{lc $cn});
	
	botserv::bot_part_if_needed($bot, {CHAN => $cn}, 'Enforcer Leaving');
}

1;
