#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::ChanReg::Close;

=head1 NAME

SrSv::ChanReg::Close - Get/set a channel's close reason.

=head1 SYNOPSIS

 use SrSv::ChanReg::Close qw(set_close get_close del_close);
 
 set_close($chan, $reason, $opernick, $type);
 $reason = get_close($chan);
 del_close($chan);

=cut

use strict;

use Exporter 'import';
BEGIN { our @EXPORT_OK = qw(set_close get_close del_close); }

use SrSv::MySQL '$dbh';
use SrSv::Process::Init;

use SrSv::ChanReg::Flags;

use SrSv::MySQL::Stub {
	set_close_real => ['NULL', "REPLACE INTO chanclose SET chan=?, reason=?, nick=?, time=UNIX_TIMESTAMP(), type=?"],
	del_close_real => ['NULL', "DELETE FROM chanclose WHERE chan=?"],
};

our ($get_close);

proc_init {
	$get_close = $dbh->prepare("SELECT reason, nick, time FROM chanclose WHERE chan=?");
};

sub get_close($) {
	my ($chan) = @_;
	my $cn;

	if(ref($chan)) {
		$cn = $chan->{CHAN};
	}
	else {
		$cn = $chan;
		$chan = { CHAN => $cn };
	}

	return undef unless cr_flag($chan, CRF_CLOSE | CRF_DRONE);

	$get_close->execute($cn);
	my ($reason, $opnick, $time) = $get_close->fetchrow_array();
	$get_close->finish();

	$reason = "[$opnick ".localtime($time)."] - $reason";
	
	return (wantarray ? ($reason, $opnick, $time) : $reason);
}

sub set_close($$$$) {
	set_close_real((ref($_[0]) ? $_[0]->{CHAN} : $_[0]), $_[1], $_[2], $_[3]);
}

sub del_close($) {
	del_close_real((ref($_[0]) ? $_[0]->{CHAN} : $_[0]));
}

1;
