#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickReg;

use strict;

use Exporter 'import';
BEGIN { our @EXPORT = qw(is_registered drop changeroot dropgroup get_all_access del_all_access) }

use SrSv::Process::Init;
use SrSv::NickReg::Alias qw(delete_aliases);

use SrSv::MySQL '$dbh';

our ($is_registered, $drop, $change_root, $get_all_access, $del_all_access);

proc_init {
	$is_registered = $dbh->prepare("SELECT 1 FROM nickalias WHERE alias=?");
	$drop = $dbh->prepare("DELETE FROM nickreg WHERE nick=?");
	$change_root = $dbh->prepare("UPDATE nickreg SET nick=? WHERE nick=?");
	$get_all_access = $dbh->prepare("SELECT chanacc.chan, chanacc.level, chanacc.adder, chanacc.time FROM nickalias, chanacc WHERE chanacc.nrid=nickalias.nrid AND nickalias.alias=? ORDER BY chanacc.chan");
	$del_all_access = $dbh->prepare("DELETE FROM chanacc USING chanacc, nickreg WHERE chanacc.nrid=nickreg.id AND nickreg.nick=?");
};

sub is_registered($) {
	my ($nick) = @_;

	$is_registered->execute($nick);
	if($is_registered->fetchrow_array) {
		return 1;
	} else {
		return 0;
	}
}

sub drop($) {
	my ($nick) = @_;
	
	return $drop->execute($nick);
}

sub changeroot($$) {
	my ($old, $new) = @_;

	return if(lc $old eq lc $new);

	$change_root->execute($new, $old);
}

sub dropgroup($) {
	my ($root) = @_;
	
	delete_aliases($root);
	$del_all_access->execute($root);
	$memoserv::delete_all_memos->execute($root);
	chanserv::drop_nick_chans($root);
	$hostserv::del_vhost->execute($root);
	drop($root);
}

sub get_all_access($) {
	my ($nick) = @_;
	
	$get_all_access->execute($nick);
	return $get_all_access->fetchall_arrayref;
}

sub del_all_access($) {
	my ($root) = @_;
	
	return $del_all_access->execute($root);
}

1;
