#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickReg::Event;

use strict;

use SrSv::Message qw(add_callback);
use SrSv::MySQL '$dbh';
use SrSv::Process::Init;
use SrSv::User qw(get_user_id);
use SrSv::NickReg::Text;

our ($quit_update);

proc_init {
	$quit_update = $dbh->prepare("UPDATE nickreg,nickid SET nickreg.last=UNIX_TIMESTAMP() WHERE nickreg.id=nickid.nrid AND nickid.id=?");
};

add_callback {
	TRIGGER_COND => { CLASS => 'SrSv::User::Event', TYPE => 'USER_QUIT' },
	CALL => __PACKAGE__ . '::user_quit',
};

sub user_quit($$) {
	my ($message) = @_;

	my ($user, $quit) = ($message->{USER}, $message->{QUIT});

	$quit_update->execute(get_user_id($user));

	update_nrt_by_nickid($user, NRT_QUIT, $quit);
}

1;
