#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickReg::Flags;

use strict;

use Exporter 'import';

BEGIN {
	my %constants = (
		NRF_HIDEMAIL => 1,
		NRF_NOMEMO => 2,
		NRF_NOACC => 4,
		NRF_NEVEROP => 8,
		NRF_AUTH => 16,
		NRF_HOLD => 32,
		NRF_FREEZE => 64,
		NRF_VACATION => 128,
	);

	our @EXPORT = (qw(nr_flag), keys(%constants));
	our @EXPORT_OK = qw(set_flag chk_flag);
	
	require constant; import constant (\%constants);
}

use SrSv::Process::Init;
use SrSv::MySQL '$dbh';

our ($get_flags, $set_flag, $unset_flag);

proc_init {
	$get_flags = $dbh->prepare("SELECT nickreg.flags FROM nickreg, nickalias WHERE nickalias.nrid=nickreg.id AND nickalias.alias=?");
	$set_flag = $dbh->prepare("UPDATE nickreg, nickalias SET nickreg.flags=IF(nickreg.flags & ?, nickreg.flags, nickreg.flags ^ ?) WHERE nickalias.nrid=nickreg.id AND nickalias.alias=?");
	$unset_flag = $dbh->prepare("UPDATE nickreg, nickalias SET nickreg.flags=IF(nickreg.flags & ?, nickreg.flags ^ ?, nickreg.flags) WHERE nickalias.nrid=nickreg.id AND nickalias.alias=?");
};

sub nr_flag($$;$) {
	my ($nick, $flag, $set) = @_;

	if(defined $set) {
		goto &set_flag;
	} else {
		goto &chk_flag;
	}
}

sub set_flag($$;$) {
	my ($nick, $flag, $sign) = @_;
	$sign = 1 unless defined($sign);

	if($sign) {
		$set_flag->execute($flag, $flag, $nick);
	} else {
		$unset_flag->execute($flag, $flag, $nick);
	}
}

sub chk_flag($$;$) {
	my ($nick, $flag, $sign) = @_;
	$sign = 1 unless defined($sign);

	$get_flags->execute($nick);
	my ($flags) = $get_flags->fetchrow_array;

	return ($sign ? ($flags & $flag) : !($flags & $flag));
}

1;
