#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickReg::Id;

use strict;

use Exporter 'import';
BEGIN { our @EXPORT_OK = qw(nr_identify nr_logout nr_unidentify is_identified get_nick_users) }

use SrSv::NickReg::User qw(is_identified get_nick_users);

use SrSv::Process::Init;
use SrSv::MySQL '$dbh';
use SrSv::NickReg::Flags;
use SrSv::NickReg::Alias qw(get_root_nick is_alias_of);
use SrSv::NickReg::User qw(get_nick_users);
use SrSv::User qw(get_user_id get_user_nick);
use SrSv::NickServ;

our ($identify, $id_update, $logout, $update_lastseen, $unidentify);

proc_init {
	$identify = $dbh->prepare("INSERT INTO nickid SELECT ?, id FROM nickreg WHERE nick=?");
	$id_update = $dbh->prepare("UPDATE nickreg, user SET
		nickreg.last=UNIX_TIMESTAMP(), nickreg.ident=user.ident,
		nickreg.vhost=user.vhost, nickreg.gecos=user.gecos,
		nickreg.nearexp=0, nickreg.flags = (nickreg.flags & ~". NRF_VACATION .")
		WHERE nickreg.nick=? AND user.id=?");
	$logout = $dbh->prepare("DELETE FROM nickid WHERE id=?");
	$update_lastseen = $dbh->prepare("UPDATE nickreg,nickid SET nickreg.last=UNIX_TIMESTAMP() WHERE nickreg.id=nickid.nrid AND nickid.id=?");
	$unidentify = $dbh->prepare("DELETE FROM nickid USING nickreg, nickid WHERE nickreg.nick=? AND nickid.nrid=nickreg.id");
};

sub nr_identify($$) {
	my ($user, $nick) = @_;
	my $uid = get_user_id($user);

	$identify->execute($uid, $nick);
	$id_update->execute($nick, $uid);
}

sub nr_logout($) {
	my ($user) = @_;
	my $uid = get_user_id($user);
	
	$update_lastseen->execute($uid);
	$logout->execute($uid);
}

# For a whole group:
sub nr_unidentify($$;$) {
	my ($nick, $msg, $src) = @_;

	$nick = get_root_nick($nick);

	my @users = get_nick_users($nick);
	foreach my $user (@users) {
		my $t = get_user_nick($user);
		
		ircd::notice($nsnick, $t, $msg) unless(lc $t eq lc $src);
		
		if(is_alias_of($nick, $t)) {
			ircd::setumode($nsnick, $t, '-r');
		}
	}

	$unidentify->execute($nick);
}

# FIXME: Is this broken?
# For a single alias:
sub nr_unidentify_single($$) {
	my ($nick, $msg) = @_;

	if(is_online($nick)) {
		ircd::setumode($nsnick, $nick, '-r');
	}

	#$unidentify_single->execute($nick);
}

1;
