#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickReg::Password;

use strict;

use Exporter 'import';
BEGIN {
	our @EXPORT = qw(chk_pass get_pass set_pass nr_pass);
}

use SrSv::MySQL qw($dbh);
use SrSv::Process::Init;
use SrSv::OnIRC;

use SrSv::MySQL::KeyValStub {
	nr_pass => [
		"SELECT nickreg.pass FROM nickreg, nickalias WHERE nickalias.nrid=nickreg.id AND nickalias.alias=?",
		"UPDATE nickreg, nickalias SET nickreg.pass=? WHERE nickalias.nrid=nickreg.id AND nickalias.alias=?",
	],
};

our ($chk_pass);

proc_init {
	$chk_pass = $dbh->prepare("SELECT 1 FROM nickreg WHERE nick=? AND pass=?");
};

sub chk_pass($$$) {
	my ($nick, $pass, $user) = @_;

	if(IRC_SERVER) {
		if(lc($pass) eq 'force' and adminserv::can_do($user, 'SERVOP')) {
			if(adminserv::get_best_svs_level($user) > adminserv::get_svs_level($nick)) {
				return 1;
			}
		}
	}

	$chk_pass->execute($nick, $pass);
	return 1 if($chk_pass->fetchrow_array);
	
	return 0;
}

# Deprecated stubs

sub get_pass($) { goto &nr_pass; }

sub set_pass($$) { goto &nr_pass; }

1;
