#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickReg::Protect;

use strict;

use Exporter 'import';
BEGIN { our @EXPORT = qw(@protect_short @protect_long nr_protect_level) }

use SrSv::MySQL qw($dbh);
use SrSv::Process::Init;

our ($protect_level, $set_protect_level);

proc_init {
	$protect_level = $dbh->prepare("SELECT protect FROM nickalias WHERE alias=?");
	$set_protect_level = $dbh->prepare("UPDATE nickalias SET protect=? WHERE alias=?");
};

our @protect_short = ('none', 'normal', 'high', 'kill');
our @protect_long = (
	'You will not be required to identify to use this nick.',
	'You must identify within 60 seconds to use this nick.',
	'You must identify before using this nick.',
	'You must identify before using this nick or you will be disconnected.'
);

sub nr_protect_level($;$) {
	my ($alias, $level) = @_;

	if(defined($level)) {
		$set_protect_level->execute($level, $alias);
	} else {
		$protect_level->execute($alias);
		($level) = $protect_level->fetchrow_array;
	}

	return $level;
}

1;
