#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickReg::Register;

use strict;

use Exporter 'import';
BEGIN { our @EXPORT_OK = qw(nr_get_register_lock nr_create_alias nr_register); }

use SrSv::Debug;

use SrSv::NickReg qw(is_registered);
use SrSv::NickReg::Id qw(nr_identify);
use SrSv::NickReg::Flags;

use SrSv::MySQL '$dbh';
use SrSv::MySQL::Unlock;
use SrSv::Process::Init;

use SrSv::User qw(get_user_nick);

use SrSv::Errors;
use SrSv::Email::Validate qw(validate_email);

# FIXME
use SrSv::NickServ;
use SrSv::User::Notice;

our ($get_register_lock, $register, $create_alias);

proc_init {
	$get_register_lock = $dbh->prepare("LOCK TABLES nickalias WRITE, nickreg WRITE");
	$register = $dbh->prepare("INSERT INTO nickreg SET nick=?, pass=?, email=?, flags=".NRF_HIDEMAIL.", regd=UNIX_TIMESTAMP(), last=UNIX_TIMESTAMP()");
	$create_alias = $dbh->prepare("INSERT INTO nickalias SELECT id, ?, NULL FROM nickreg WHERE nick=?");
};

sub nr_get_register_lock($) {
	if($_[0]) {
		$get_register_lock->execute; $get_register_lock->finish;
		print "register locked.\n" if DEBUG;
	} else {
		$unlock_tables->execute; $unlock_tables->finish;
		print "register unlocked.\n" if DEBUG;
	}
}

sub nr_create_alias($$) {
	my ($root, $alias) = @_;

	$create_alias->execute($alias, $root);
}

sub nr_register($$$) {
	my ($user, $pass, $email) = @_;
	
	my $src = get_user_nick($user);
	
	if($src =~ /^guest/i) {
		notice($user, $err_deny);
		return;
	}
	
	unless(validate_email($email)) {
		notice($user, $err_email);
		return;
	}
	
	nr_get_register_lock 1;
	
	if(not is_registered($src)) {
		$register->execute($src, $pass, $email);
		$create_alias->execute($src, $src);

		nr_get_register_lock 0;
		
		nr_identify($user, $src);
		notice($user, 'You are now registered and identified.');
		ircd::setumode($nsnick, $src, '+r');
		services::ulog($nsnick, log::INFO(), "registered $src (email: $email)", $user);
	} else {
		nr_get_register_lock 0;
		notice($user, 'Your nickname has already been registered.');
	}
}

1;
