#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickReg::Seen;

use strict;

use Exporter 'import';
BEGIN { our @EXPORT = qw(do_seen) }

use SrSv::Process::Init;
use SrSv::MySQL '$dbh';
use SrSv::NickReg::User qw(get_nick_users);
use SrSv::User qw(get_user_nick);

our ($get_seen);

proc_init {
	$get_seen = $dbh->prepare("SELECT nickalias.alias, nickreg.nick, nickreg.last FROM nickreg, nickalias 
		WHERE nickalias.nrid=nickreg.id AND nickalias.alias=?");
};

sub do_seen($) {
	my ($nick) = @_;
	my ($status, $msg);
	
	$get_seen->execute($nick);
	if (my ($alias, $root, $lastseen) = $get_seen->fetchrow_array) {
		my @users = get_nick_users($root);
		if(@users) {
			my @usernicks = map get_user_nick($_), @users;
			
			$status = 2;
			$msg = "using ".(@usernicks==1 ? 'the nick ' : 'the following nicks: ').join(', ', @usernicks);
		}
		else {
			$status = 1;
			$msg = misc::time_ago($lastseen) . " ago (".scalar(localtime($lastseen)).")";
		}
	}
	else {
		$status = 0; $msg = undef();
	}

	return ($status, $msg);
}

1;
