#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickReg::Text;

use strict;

use Exporter 'import';

BEGIN {
	my %types = (
		NRT_QUIT => 1,
		NRT_GREET => 2,
	);

	our @EXPORT = (qw(update_nrt update_nrt_by_server update_nrt_by_nickid get_nrt get_nrt_by_nickid), keys(%types));
	our %EXPORT_TAGS = (types => [keys(%types)]);
	
	require constant; import constant (\%types);
}

use SrSv::MySQL '$dbh';
use SrSv::Process::Init;
use SrSv::User qw(get_user_id);

our ($update_nrt, $delete_nrt, $server_update_nrt, $nickid_update_nrt, $get_nrt, $nickid_get_nrt);

proc_init {
	$update_nrt = $dbh->prepare("REPLACE INTO nickregtext SELECT nickalias.nrid, ?, ? FROM nickalias WHERE nickalias.alias=?");
	$delete_nrt = $dbh->prepare("DELETE FROM nickregtext USING nickregtext, nickalias WHERE nickregtext.nrid=nickalias.nrid AND nickalias.alias=? AND nickregtext.type=?");
	$server_update_nrt = $dbh->prepare("REPLACE INTO nickregtext SELECT nickid.nrid, ?, ? FROM nickid, user WHERE nickid.id=user.id AND user.online=1 AND user.server=?");
	$nickid_update_nrt = $dbh->prepare("REPLACE INTO nickregtext SELECT nickid.nrid, ?, ? FROM nickid WHERE nickid.id=?");
	$get_nrt = $dbh->prepare("SELECT nickregtext.text FROM nickalias, nickregtext WHERE nickalias.alias=? AND nickregtext.nrid=nickalias.nrid AND nickregtext.type=?");
	$nickid_get_nrt = $dbh->prepare("SELECT nickregtext.text FROM nickid, nickregtext WHERE nickid.id=? AND nickregtext.nrid=nickid.nrid AND nickregtext.type=? LIMIT 1");
};

sub update_nrt($$$) {
	my ($nick, $type, $text) = @_;
	if(defined($text)) {
		$update_nrt->execute($type, $text, $nick);
	} else {
		$delete_nrt->execute($nick, $type);
	}
}

sub update_nrt_by_server($$$) {
	my ($server, $type, $text) = @_;

	$server_update_nrt->execute($type, $text, $server);
}

sub update_nrt_by_nickid($$$) {
	my ($user, $type, $text) = @_;

	$nickid_update_nrt->execute($type, $text, get_user_id($user));
}

sub get_nrt($$) {
	my ($nick, $type) = @_;

	$get_nrt->execute($nick, $type);
	return $get_nrt->fetchrow_array;
}

sub get_nrt_by_nickid($$) {
	my ($user, $type) = @_;

	$nickid_get_nrt->execute(get_user_id($user), $type);
	return $nickid_get_nrt->fetchrow_array;
}

1;
