#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickServ::Alist;

use strict;

use Exporter 'import';
BEGIN { our @EXPORT = qw(ns_alist) }

use SrSv::NickServ::UI;
use SrSv::MySQL qw($dbh);
use SrSv::Process::Init;
use SrSv::User qw(get_user_nick);
use SrSv::User::Notice;

use SrSv::NickReg qw(get_all_access);
use SrSv::NickReg::IRC qw(chk_registered);

ns_command {
	SYNTAX => 'alist [LIST:nicks]',
	CALL => \&ns_alist,
};

sub ns_alist($$) {
	my ($user, $targets) = @_;
	my @reply;

	$targets = [get_user_nick($user)] unless @$targets;

	foreach my $target (@$targets) {
		(adminserv::is_svsop($user, adminserv::S_HELP()) and (
			chk_registered($user, $target) or next)
		) or chk_identified($user, $target) or next;

		my @data;

		for (@{get_all_access($target)}) {
			my ($c, $l, $a, $t) = @$_;
			push @data, [$c, $chanserv::levels[$l], ($a ? "($a)" : ''), ($t ? scalar(localtime($t)) : '')];
		}

		push @reply,
			"Access listing for \002$target\002 (".
			scalar(@data)." entries)",
			fmt::columnar(@data) ;
	}
	notice($user, @reply);
}

1;
