#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickServ::Glist;

use strict;

use SrSv::NickServ::UI;
use SrSv::User qw(get_user_nick);
use SrSv::User::Notice;
use SrSv::Errors;
use SrSv::NickReg::Id qw(is_identified);
use SrSv::NickReg::Alias qw(get_root_nick get_aliases);

ns_command {
	SYNTAX => 'glist/links [LIST:nicks]',
	CALL => \&ns_glist,
};

sub ns_glist($$) {
	my ($user, $targets) = @_;
	my @reply;

	$targets = [get_user_nick($user)] unless (@$targets);

	foreach my $target (@$targets) {
		unless(is_identified($user, $target) or 
			adminserv::is_svsop($user, adminserv::S_HELP())
		) {
			push @reply, "$target: $err_deny";
			next;
		}

		my $root = get_root_nick($target);
		unless($root) {
			push @reply, "\002$target\002 is not registered.";
			next;
		}

		my @data = @{ get_aliases($root) };
		
		push @reply,
			"The following ". scalar(@data) ." nicks are aliases of \002$root\002:",
			fmt::columnar(@data);
	}

	notice($user, @reply);
}

1;
