#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickServ::Info;

use strict;

use Exporter 'import';
BEGIN { our @EXPORT = qw(ns_info) }

use SrSv::NickServ::UI;
use SrSv::MySQL qw($dbh);
use SrSv::Process::Init;
use SrSv::User::Notice;
use SrSv::NickReg::Alias qw(get_root_nick);
use SrSv::NickReg::Id qw(is_identified);
use SrSv::NickReg::Seen qw(do_seen);
use SrSv::NickReg::Flags;
use SrSv::NickReg::Protect qw(@protect_long nr_protect_level);
use SrSv::NickReg::Text;

our $get_info;

#FIXME: Move mysql stuff into its own module
proc_init {
	$get_info = $dbh->prepare("SELECT nickreg.email, nickreg.regd, nickreg.last, nickreg.flags, nickreg.ident, nickreg.vhost, nickreg.gecos FROM nickreg, nickalias WHERE nickalias.nrid=nickreg.id AND nickalias.alias=?");
};

ns_command {
	SYNTAX => 'info LIST:nicks',
	CALL => \&ns_info,
};

sub ns_info($$) {
	my ($user, $nicks) = @_;
	my @reply;

	foreach my $nick (@$nicks) {
		my $root = get_root_nick($nick);
	
		$get_info->execute($root);
		my @result = $get_info->fetchrow_array;
		$get_info->finish();

		unless(@result) {
			notice($user, "The nick \002$nick\002 is not registered.");
			return;
		}
	
		my ($email, $regd, $last, $flags, $ident, $vhost, $gecos) = @result;
		my $quit = get_nrt($nick, NRT_QUIT);
		my $hidemail = $flags & NRF_HIDEMAIL;

		my $svslev = adminserv::get_svs_level($root);
		my $protect = nr_protect_level($nick);
		my $showprivate = (is_identified($user, $nick) or
			adminserv::is_svsop($user, adminserv::S_HELP()));
	
		my ($seens, $seenm) = do_seen($nick);

		my @data;
		push @reply, "NickServ info for \002$nick\002:";
		
		push @reply, "  (Online now, $seenm.)" if $seens == 2;
		push @data, ["Last seen:", "$seenm."] if $seens == 1;
		
		push @data,
			["Last seen address:", "$ident\@$vhost"],
			["Registered:", scalar(localtime($regd))],
			["Last real name:", $gecos],
			["Last quit:", $quit];
		
		push @data, ["Services Rank:", $adminserv::levels[$svslev]] 
			if $svslev;
		push @data, ["E-mail:", $email] unless $hidemail;
		push @data, ["E-mail:", "$email (Hidden)"] 
			if($hidemail and $showprivate);
		push @data, ["Alias of:", $root] 
			if ((lc $root ne lc $nick) and $showprivate);

		push @reply, fmt::columnar(@data);
		pop @reply; push @reply, ' ';

		push @reply, $protect_long[$protect] if $protect;
		push @reply, "Does not accept memos." if($flags & NRF_NOMEMO);
		push @reply, "Cannot be added to channel access lists." if($flags & NRF_NOACC);
		push @reply, "Requires authorization to be added to channel access lists."
			if($flags & NRF_AUTH);
		push @reply, "Is frozen and may not be used." if($flags & NRF_FREEZE);
		push @reply, "Will not expire." if($flags & NRF_HOLD);
		push @reply, "Is currently on vacation." if($flags & NRF_VACATION);

		pop @reply if($reply[-1] eq ' ');
		push @reply, ' --';
	}

	notice($user, @reply);
}

1;
