#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickServ::Link;

use strict;

use SrSv::NickReg qw(is_registered);
use SrSv::NickReg::Alias qw(is_alias_of get_root_nick);
use SrSv::NickReg::Id qw(is_identified);
use SrSv::NickReg::Register qw(nr_get_register_lock nr_create_alias);
use SrSv::NickReg::Password qw(chk_pass);

use SrSv::NickServ;
use SrSv::NickServ::Enforcer qw(check_identify);
use SrSv::NickServ::Id qw(ns_identify);

use SrSv::User qw(get_user_nick);

use SrSv::NickServ::UI;
use SrSv::User::Notice;
use SrSv::Errors;

ns_command {
	SYNTAX => 'link/group nick password',
	CALL => \&ns_link,
};

sub ns_link($$$) {
	my ($user, $nick, $pass) = @_;

	my $root = get_root_nick($nick);
	my $src = get_user_nick($user);
	#my $uid = get_user_id($user);

	if($src =~ /^guest/i) {
		notice($user, $err_deny);
		return;
	}

	unless($root) {
		if(is_registered($src)) {
			notice($user, "The nick \002$nick\002 is not registered. You need to change your nick to \002$nick\002 and then link to \002$src\002.");
		} else { # if neither $nick nor $src are registered
			notice($user, "You need to register your nick first. For help, type \002/ns help register");
		}
		return;
	}

	unless(chk_pass($root, $pass, $user)) {
		notice($user, $err_pass);
		return;
	}
	
	if(is_alias_of($src, $nick)) {
		notice($user, "\002$nick\002 is already linked to \002$src\002.");
		return;
	}

	nr_get_register_lock 1;
		
	if(is_registered($src)) {
		nr_get_register_lock 0;
		
		if(is_identified($user, $src)) {
			# This is disabled because it's not used much, it's easy to abuse, and it causes inconsistencies in nickid and chanacc.
			
			notice($user, "You cannot link an already registered nick. Type this and try again: \002/ns drop $src <password>");
			return;
			
			#my $sroot = get_root_nick($src);
			
			## FIXME - This may cause duplicates on access lists
			
			#$change_root->execute($root, $sroot);
			#$change_all_access->execute($root, $sroot);
			#$change_akicks->execute($root, $sroot);
			#$change_founders->execute($root, $sroot);
			#$change_successors->execute($root, $sroot);
			#$change_svsops->execute($root, $sroot);

			#$memoserv::memo_chgroot->execute($root, $sroot);
			#$memoserv::count_ignore->execute($sroot);
			#my $count = $memoserv::count_ignore->fetchrow_array;
			#$memoserv::adjust_ignore->execute($count, $sroot);
			#$memoserv::ignore_chgroot1->execute($root, $sroot);
			#$memoserv::ignore_chgroot2->execute($root, $sroot);

			#$drop->execute($sroot);

			#services::ulog($nsnick, log::INFO(), "merged nicks $sroot and $root.", $user);
		} else {
			notice($user, 'Your nickname has already been registered.');
			return;
		}
	} else {
		nr_create_alias($root, $src);
		nr_get_register_lock 0;
		
		if(is_identified($user, $root)) {
			#$identify_ign->execute($uid, $root);
			#$id_update->execute($root, $uid);
		} else {
			ns_identify($user, $root, $pass);
		}
	}
	
	notice($user, "\002$src\002 is now linked to \002$root\002.");
	services::ulog($nsnick, log::INFO(), "made $src an alias of $root.", $user);

	check_identify($user);
}

1;
