#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickServ::Register;

use strict;

use SrSv::NickReg::Register qw(nr_register);
use SrSv::NickServ::UI;
use SrSv::User::Notice;
use SrSv::Errors;

ns_command {
	SYNTAX => 'register password email',
	CALL => \&ns_register,
};

sub ns_register($$$) {
	my ($user, $pass, $email) = @_;

	nr_register($user, $pass, $email);
}

=cut

sub ns_unlink($$$) {
	my ($user, $nick, $pass) = @_;
	my $uid = get_user_id($user);
	my $src = get_user_nick($user);
	
	my $root = get_root_nick($nick);
	unless(chk_pass($root, $pass, $user)) {
		notice($user, $err_pass);
		return;
	}

	if(lc $root eq lc $nick) {
		my ($count) = count_aliases($root);
		if($count == 1) {
			ns_dropgroup_real($user, $root);
			return;
		}

		$get_random_alias->execute($root);
		my ($new) = $get_random_alias->fetchrow_array;
		ns_changeroot($user, $root, $new, 1);
		
		$root = $new;
	}
	
	unidentify_single($nick);
	delete_alias($nick);
	
	notice($user, "\002$nick\002 has been unlinked from \002$root\002.");
	services::ulog($nsnick, log::INFO(), "removed alias $nick from $root.", $user);
}

sub ns_dropgroup($$$) {
	my ($user, $nick, $pass) = @_;
	my $uid = get_user_id($user);
	my $src = get_user_nick($user);
	my $root = get_root_nick($nick);

	if(adminserv::get_svs_level($root)) {
		notice($user, "A nick with services access may not be dropped.");
		return;
	}

	unless(chk_pass($root, $pass, $user)) {
		notice($user, $err_pass);
		return;
	}

	ns_dropgroup_real($user, $root);
}

sub ns_dropgroup_real($$) {
	my ($user, $root) = @_;
	my $src = get_user_nick($user);
	
	nr_unidentify($root, "Your nick, \002$root\002, was dropped by \002$src\002.", $src);
	dropgroup($root);
	notice($user, "Your nick(s) have been dropped.  Thanks for playing.");
	
	services::ulog($nsnick, log::INFO(), "dropped group $root.", $user);
}

sub ns_changeroot($$$;$) {
	my ($user, $old, $new, $force) = @_;

	$force or chk_identified($user, $old) or return;

	my $root = get_root_nick($old);
	
	#unless($oldroot =~ /^\Q$rootnick\E$/i) {
	#	notice($user, "\002$oldroot\002 is not a root nick.  Did you mean \002$rootnick\002?");
	#	return;
	#}

	if(lc($new) eq lc($root)) {
		notice($user, "\002$root\002 is already your root nick.");
		return;
	}
	
	unless(get_root_nick($new) eq $root) {
		notice($user, "\002$new\002 is not an alias of your nick.  Type \002/msg nickserv help link\002 for information about creating aliases.");
		return;
	}

	changeroot($root, $new);

	notice($user, "Your root nick is now \002$new\002.");
	services::ulog($nsnick, log::INFO(), "changed root $root to $new.", $user);
}

=cut

1;
