#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickServ::SendPass;

use strict;

use SrSv::NickServ;
use SrSv::NickServ::UI;

use SrSv::Process::Init;
use SrSv::MySQL '$dbh';

use SrSv::User::Notice;
use SrSv::Errors;

our $get_pass_email;

proc_init {
	$get_pass_email = $dbh->prepare("SELECT nickreg.pass, nickreg.email FROM nickalias, nickreg WHERE nickreg.id=nickalias.nrid AND nickalias.alias=?");
};

ns_command {
	SYNTAX => 'sendpass nick',
	CALL => \&ns_sendpass,
};

sub ns_sendpass($$) {
	my ($user, $nick) = @_;

	unless(adminserv::is_svsop($user, adminserv::S_HELP() )) {
		notice($user, $err_deny);
		return;
	}

	$get_pass_email->execute($nick);
	my ($pass, $email) = $get_pass_email->fetchrow_array;

	unless($pass and $email) {
		notice($user, "\002$nick\002 is not registered or does not have an email address.");
		return;
	}

	email::send($email, "$nsnick Password Reminder",
		"The password for the nick $nick is:\n$pass");
	
	notice($user, "Password has been sent to $email.");
	services::ulog($nsnick, log::INFO(), "used SENDPASS on $nick ($email)", $user);
}

1;
