#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickServ::Set;

use strict;

use Exporter 'import';
BEGIN { our @EXPORT = qw(ns_set ns_setting) }

use SrSv::NickServ::UI;
use SrSv::NickReg::Id qw(is_identified);
use SrSv::User qw(get_user_nick);
use SrSv::User::Notice;
use SrSv::Errors;

our @ns_set_list;

sub ns_setting(@) {
	push @ns_set_list, @_;
}

ns_command {
	SYNTAX => 'set [nick] setting value',
	OPTIONAL_DEFAULT_SRC => 1,
	CALL => \&ns_set,
};

sub ns_set($$$$) {
	my ($user, $target, $set, $param) = @_;

	my $call;
	unless ($call = get_match(\@ns_set_list, $set)) {
		notice($user, qq{"$set" is not a valid NickServ setting.}, "For help, type: \002$ui->{PREFIX} help set");
		return;
	}

	#my $override = adminserv::can_do($user, 'SERVOP') or
	#	(adminserv::can_do($user, 'FREEZE') and $set =~ /^freeze$/i);
	
	my $override;
	
	if($call->{REQUIRE}) {
		unless ($override = adminserv::can_do($user, $call->{REQUIRE}) or
			is_identified($user, $target) and adminserv::is_ircop($user))
		{	
			notice($user, $err_deny);
			return;
		}
	}
	else {
		$call->{OVERRIDE} = 'SERVOP' unless defined($call->{OVERRIDE});
		
		unless(is_identified($user, $target) or ($override = adminserv::can_do($user, $call->{OVERRIDE}))) {
			notice($user, $err_deny);
			return;
		}
	}

	my $p = {
		USER => $user,
		SRC => get_user_nick($user),
		TARGET => $target,
		SET => $set,
		PARAM => $param,
		OVERRIDE => $override,
	};
	
	if($p->{SRC} eq $target) {
		$p->{SUBJ}='Your';
		$p->{OBJ}='You';
	} else {
		$p->{SUBJ}="\002$target\002\'s";
		$p->{OBJ}="\002$target\002";
	}

	if(defined($call->{BOOL})) {
		if($param =~ /^(?:no|off|false|0)$/i) { $p->{VAL} = 0; }
		elsif($param =~ /^(?:yes|on|true|1)$/i) { $p->{VAL} = 1; }
		else {
			notice($user, "Please say \002on\002 or \002off\002.");
			return;
		}
	}
	
	$call->{CALL}->($p);
}

sub get_match($$) {
	my ($list, $str) = @_;

	foreach my $elem (@$list) {
		return $elem->[1] if $str =~ ($elem->[0]);
	}

	return undef;
}

1;
