#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickServ::Settings;

use strict;

use SrSv::NickServ;
use SrSv::NickServ::Set;
use SrSv::User::Notice;
use SrSv::Errors;

use SrSv::NickReg::Values qw(nr_email);
use SrSv::NickReg::Password qw(nr_pass);
use SrSv::NickReg::Flags;

use SrSv::Email::Validate 'validate_email';

# TODO: Possibly split these up more.

ns_setting [qr/^e?-?mail$/i, {NAME=>'EMAIL', CALL => \&ns_set_email, OVERRIDE=>'HELP'}];

sub ns_set_email($) {
	my ($p) = @_;
	
	unless($p->{PARAM} =~ /^\S+$/) {
		notice($p->{USER}, 'Syntax: SET EMAIL <address>');
		return;
	}
	my $email = $p->{PARAM};
	
	unless(validate_email($email)) {
		notice($p->{USER}, $err_email);
		return;
	}

	nr_email($p->{TARGET}, $email);
	
	notice($p->{USER}, "$p->{SUBJ} email address has been changed to \002$email\002.");
	services::ulog($nsnick, log::INFO(), "changed email of \002$p->{TARGET}\002 to $email", $p->{USER});
}


ns_setting [qr/^pass(?:w(?:or)?d)?$/i, {NAME=>'PASSWORD', CALL => \&ns_set_password, OVERRIDE=>'SERVOP'}];

sub ns_set_password($) {
	my ($p) = @_;
	
	unless($p->{PARAM} =~ /^\S+$/) {
		notice($p->{USER}, 'Syntax: SET PASSWORD <password>');
		return;
	}

	nr_pass($p->{TARGET}, $p->{PARAM});
	
	notice($p->{USER}, "$p->{SUBJ} password has been changed.");
	services::ulog($nsnick, log::INFO(), "changed password of \002$p->{TARGET}\002", $p->{USER});
}

1;
