#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickServ::Settings::Boolean;

use strict;

use SrSv::NickServ::Set;
use SrSv::User::Notice;
use SrSv::NickReg::Flags;

sub create_bool_setting($) {
	my ($name, $flag, $noun_type, $on_text, $off_text) = @{$_[0]}[1..5];

	return sub ($) {
		my ($p) = @_;

		nr_flag($p->{TARGET}, $flag, $p->{VAL});

		notice($p->{USER},
			"\002$name\002 is now ". ($p->{VAL} ? 'on' : 'off'). '.  '.
			$p->{$noun_type}. ' '. ($p->{VAL} ? $on_text : $off_text)
		);
	}
}

{
	my @settings = (
		[qr/^hidee?-?mail$/i, 'HIDEMAIL', NRF_HIDEMAIL, 'SUBJ', "email address is now hidden.", "email address is now visible."],
		[qr/^nomemo$/i, 'NOMEMO', NRF_NOMEMO, 'SUBJ', "memos will be blocked.", "memos will be delivered."],
		[qr/^no(?:acc|op)$/i, 'NOACC', NRF_NOACC, 'OBJ', "may not be added to channel access lists.",
			"may be added to channel access lists."],
		[qr/^neverop$/i, 'NEVEROP', NRF_NEVEROP, 'OBJ', "will not be granted status upon joining channels.",
			"will be granted status upon joining channels."],
		[qr/^auth$/i, 'AUTH', NRF_AUTH, 'OBJ', "must now authorize additions to channel access lists.",
			"will not be asked to authorize additions to channel access lists."],
	);
		
	foreach my $setting (@settings) {
		ns_setting [$setting->[0], {NAME=>$setting->[1], CALL => create_bool_setting($setting), BOOL=>1}];
	}

	undef @settings;
}

1;
