#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickServ::Settings::Oper;

use strict;

use SrSv::NickServ;
use SrSv::NickServ::Set;
use SrSv::User::Notice;
use SrSv::NickReg::Flags;

use SrSv::NickReg::Id qw(nr_unidentify);

ns_setting [qr/^(hold|no-?expire)$/i, {NAME=>'HOLD', CALL => \&ns_set_hold, BOOL=>1, REQUIRE=>'SERVOP'}];

sub ns_set_hold($) {
	my ($p) = @_;
	
	nr_flag($p->{TARGET}, NRF_HOLD, $p->{VAL});

	if($p->{VAL}) {
		notice($p->{USER}, "\002$p->{TARGET}\002 is now held from expiration.");
		services::ulog($nsnick, log::INFO(), "has held \002$p->{TARGET}\002", $p->{USER});
	} else {
		notice($p->{USER}, "\002$p->{TARGET}\002 will now expire normally.");
		services::ulog($nsnick, log::INFO(), "released \002$p->{TARGET}\002 from hold", $p->{USER});
	}
}


ns_setting [qr/^freeze$/i, {NAME=>'FREEZE', CALL => \&ns_set_freeze, BOOL=>1, REQUIRE=>'FREEZE'}];

sub ns_set_freeze($) {
	my ($p) = @_;

	nr_flag($p->{TARGET}, NRF_FREEZE, $p->{VAL});

	if($p->{VAL}) {
		notice($p->{USER}, "\002$p->{TARGET}\002 is now frozen.");
		nr_unidentify($p->{TARGET}, "Your nick, \002$p->{TARGET}\002, has been frozen and may no longer be used.");
		services::ulog($nsnick, log::INFO(), "froze \002$p->{TARGET}\002", $p->{USER});
	} else {
		notice($p->{USER}, "\002$p->{TARGET}\002 is no longer frozen.");
		services::ulog($nsnick, log::INFO(), "unfroze \002$p->{TARGET}\002", $p->{USER});
	}
}

1;
