#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickServ::Settings::Vacation;

use strict;

use SrSv::NickServ;
use SrSv::NickServ::Set;
use SrSv::NickReg::Time qw(nr_regd_time);
use SrSv::NickReg::Flags;
use SrSv::Conf qw(services);
use SrSv::User::Notice;

ns_setting [qr/^vacation$/i, {NAME=>'VACATION', CALL=>\&ns_set_vacation, BOOL=>1}];

sub ns_set_vacation($) {
	my ($p) = @_;
	
	my $time_remaining = (nr_regd_time($p->{TARGET}) + 86400 * int($services_conf{'vacationexpire'} / 3)) - time();

	if($p->{VAL} and $time_remaining > 0 and !$p->{OVERRIDE}) {
		notice($p->{USER}, "\002$p->{TARGET}\002 is not old enough to go on vacation.",
			'Try again in '. misc::time_rel($time_remaining). '.');
		return;
	}

	nr_flag($p->{TARGET}, NRF_VACATION, $p->{VAL});

	services::ulog($nsnick, log::INFO(),
		($p->{VAL} ? 'enabled' : 'disabled')." vacation mode for \002$p->{TARGET}\002", $p->{USER});
	notice $p->{USER}, "Vacation mode ".($p->{VAL} ? 'enabled' : 'disabled')." for \002$p->{TARGET}\002.";
}

1;
