#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# TODO: Allow more than one request per connection.

package SrSv::SocketServer;

use strict;

use Exporter 'import';
BEGIN { our @EXPORT = qw(ss_open) }

use Fcntl qw(F_GETFL F_SETFL O_NONBLOCK);
use IO::Socket::UNIX;
use Storable qw(freeze thaw);

#use SrSv::Message qw(message add_callback);
use SrSv::Process::Call qw(safe_call);

our $socket;

# The message stuff didn't work out; do everything in the parent.
#add_callback {
#	TRIGGER_COND => { CLASS => __PACKAGE__, TYPE => 'NEW' },
#	CALL => __PACKAGE__ . '::ss_accept'
#};

sub ss_accept {
	my $conn = $socket->accept or die $!;
	$conn->connected or return;

	my $flags = fcntl($conn, F_GETFL, 0);
	$flags = fcntl($conn, F_SETFL, $flags | O_NONBLOCK);

	my ($in, $args);

	do {
		while($conn->sysread(my $part, 4096)) {
			$in .= $part;
		}

		$args = thaw($in);
	} until $args;

	print "ARGS: ".join(', ', @$args)."\n";
	my $call = shift @$args;

	my $reply = freeze([1, safe_call($call, $args->[0])]);

	$conn->syswrite($reply);

	$conn->shutdown(2);
	$conn->close;
}

#sub ss_connection {
#	message {
#		CLASS => __PACKAGE__,
#		TYPE => 'NEW',
#	};
#}

sub ss_open {
	if(-S 'socket') {
		unlink 'socket';
	}

	$socket = IO::Socket::UNIX->new(
		Type => SOCK_STREAM,
		Local => 'socket',
		Peer => 'socket',
		Listen => 128,
		Blocking => 0,
	) or die $!;

	#$socket->listen(128);

	event::add_io_watcher(
		cb => \&ss_accept,
		fd => $socket,
	);
}

1;
