#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::User::Clones;

use strict;

use Exporter 'import';
BEGIN { our @EXPORT_OK = qw() }

use SrSv::Process::Init;
use SrSv::MySQL '$dbh';
use SrSv::Message qw(add_callback message);
use SrSv::NickServ;
use SrSv::User qw(:flags get_user_id get_user_nick);
use SrSv::Conf 'services';

use constant {
	MAX_LIM => 16777215,
};

our ($chk_clone_except, $count_clones);

proc_init {
	$chk_clone_except = $dbh->prepare("SELECT GREATEST(IF((user.ip >> (32 - sesexip.mask)) = (sesexip.ip >> (32 - sesexip.mask)), sesexip.lim, 0), IF(IF(sesexname.serv, user.server, user.host) LIKE sesexname.host, sesexname.lim, 0)) AS n FROM user, sesexip, sesexname WHERE user.id=? ORDER BY n DESC LIMIT 1");
	$count_clones = $dbh->prepare("SELECT COUNT(*) FROM user WHERE ip=? AND (flags & ".UF_ONLINE.")");
};

add_callback {
	TRIGGER_COND => {
		CLASS => 'SrSv::User::Host',
		TYPE => 'USERIP',
	},
	CALL => __PACKAGE__ . '::userip',
};

sub userip($) {
	my ($message) = @_;

	if(kill_clones($message->{USER}, $message->{IIP})) {
		$message->{KILLED} = 1;
	}
}

sub kill_clones($$) {
	my ($user, $ip) = @_;
	
	my $uid = get_user_id($user);
	my $src = get_user_nick($user);

	return 0 if $ip == 0;

	$chk_clone_except->execute($uid);
	my ($lim) = $chk_clone_except->fetchrow_array;
	return 0 if $lim == MAX_LIM;
	$lim = $services_conf{'clone-limit'} unless $lim;
	
	$count_clones->execute($ip);
	my ($c) = $count_clones->fetchrow_array;

	if($c > $lim) {
		ircd::irckill($nsnick, $src, "Session Limit Exceeded");
		return 1;
	}
}

1;
