#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::User::Event;

use strict;

#use Exporter 'import';
#BEGIN { our @EXPORT_OK = qw() }

use SrSv::User::Host qw(userip);
use SrSv::User::Clones;

use SrSv::Debug;
use SrSv::User qw(:flags get_user_nick get_user_id $get_user_id);
use SrSv::User::Guest qw(guestnick $get_guest $set_guest);
use SrSv::IRCd::Event qw(addhandler);
use SrSv::IRCd::State qw($ircline synced);
use SrSv::Process::Init;
use SrSv::MySQL '$dbh';
use SrSv::Process::InParent qw(olduser newuser);
use SrSv::MySQL::Unlock;
use SrSv::Message qw(add_callback message);

# TODO: This should not rely on NickServ.
use SrSv::NickServ;
use SrSv::NickServ::Enforcer qw(%enforcers check_identify);

our (%olduser, %newuser);

addhandler('NICKCONN', undef, undef, 'ev_nickconn');
addhandler('NICKCHANGE', undef, undef, 'nick_change');
addhandler('QUIT', undef, undef, 'user_quit');
addhandler('SQUIT', undef, undef, 'squit') if $ircd::noquit;

sub ev_nickconn {
    if($enforcers{@_[0]}) {
	# Scream real loud about a collision
	# send a KILL
	# quit our agent and delete its timer
	#
	#btw this is more reason for there to be a kill handler for all agents!
    }
    
    user_create(@_[0,2..4,8,5,7,9,10]);
}

sub olduser($;$) {
	if(defined($_[1])) {
		print $_[0] . " is now an olduser\n" if DEBUG;
		return $olduser{$_[0]} = 1;
	} else {
		my $old = defined($olduser{$_[0]});
		delete $olduser{$_[0]};
		print $_[0] . " olduser " . $old . "\n" if DEBUG;
		return ($old ? 1 : ());
	}
}

sub newuser($;$) {
	if(defined($_[1])) {
		print $_[0] . " is now a newuser\n" if DEBUG;
		return $newuser{$_[0]} = 1;
	} else {
		my $new = defined($newuser{$_[0]});
		delete $newuser{$_[0]};
		print $_[0] . " newuser " . $new . "\n" if DEBUG;
		return ($new ? 1 : ());
	}
}

our ($user_check, $user_create, $user_create_old, $add_nickchg);

proc_init {
	$user_check = $dbh->prepare("SELECT id FROM user WHERE nick=? AND !(flags & ".UF_ONLINE().") AND time=?");
	$user_create = $dbh->prepare("INSERT INTO user SET id=(RAND()*294967293)+1, nick=?, time=?, inval=0, ident=?,
		host=?, vhost=?, server=?, modes=?, gecos=?, flags=?");
	$user_create_old = $dbh->prepare("UPDATE user SET ident=?, host=?, vhost=?, server=?, modes=?, gecos=?,
		flags=(flags | ?) WHERE nick=?");
	
	$add_nickchg = $dbh->prepare("REPLACE INTO nickchg SELECT ?, id, ? FROM user WHERE nick=?");
};

sub user_create {
	my ($nick, $time, $ident, $host, $vhost, $server, $modes, $gecos, $ip) = @_;
	my $user = { NICK => $nick };
	
	$user_check->execute($nick, $time);
	my ($id) = $user_check->fetchrow_array;

	if($id) {
		olduser lc $nick, 1;
		$user_create_old->execute($ident, $host, $vhost, $server, $modes, $gecos, UF_ONLINE() | UF_FINISHED(), $nick);
	} else {
		nick_cede($nick);
		
		my $flags = (synced() ? UF_ONLINE() | UF_FINISHED() : UF_ONLINE() );
		my $i;
		while($i < 10 and !$user_create->execute($nick, $time, $ident, $host, $vhost, $server, $modes, $gecos, $flags)) { $i++ }
	}

	$add_nickchg->execute($ircline, $nick, $nick);

	if($vhost eq '*') {
		ircd::userhost($nick);
	}
	
	newuser lc $nick, 1;

	if($ip) {
		userip(undef, $nick, $ip);
	}
	else {
		ircd::userip($nick);
	}

	return $id;
}

our ($user_delete, $nick_id_delete);

proc_init {
	$user_delete = $dbh->prepare("DELETE FROM user WHERE nick=?");
	$nick_id_delete = $dbh->prepare("DELETE FROM nickid WHERE id=?");
};

sub nick_cede($) {
	my ($nick) = @_;
	my $id;

	$get_user_id->execute($nick);
	if($id = $get_user_id->fetchrow_array) {
		$nick_id_delete->execute($id);
		$user_delete->execute($nick);
	}
}

sub nick_create_post($) {
	my ($message) = @_;

	return if $message->{KILLED};
	
	my $user = $message->{USER};
	my $nick = get_user_nick($user);

	return unless newuser lc $nick;
	
	my $old = olduser lc $nick;
	print "old is $old\n" if DEBUG;

	message {
		CLASS => __PACKAGE__,
		TYPE => 'NEW_USER',
		USER => $user,
		OLD => $old,
		SYNC => 1,
	};
}

our ($del_nickchg_id, $quit_update, $get_quit_empty_chans, $nick_chan_delete, $user_quit);

proc_init {
	$del_nickchg_id = $dbh->prepare("DELETE FROM nickchg WHERE nickid=?");
	$get_quit_empty_chans = $dbh->prepare("SELECT cu2.chan, COUNT(*) AS c
		FROM chanuser AS cu1, chanuser AS cu2
		WHERE cu1.nickid=?
		AND cu1.chan=cu2.chan AND cu1.joined=1 AND cu2.joined=1
		GROUP BY cu2.chan HAVING c=1 ORDER BY NULL");
	$nick_chan_delete = $dbh->prepare("DELETE FROM chanuser WHERE nickid=?");
	$user_quit = $dbh->prepare("UPDATE user SET flags=(flags & ~".UF_ONLINE()."), quittime=UNIX_TIMESTAMP()
		WHERE nick=?");
};

sub user_quit($$) {
	my ($nick, $quit) = @_;
	my $user = { NICK => $nick };
	
	my $id = get_user_id($user);

	$del_nickchg_id->execute($id);

	message {
		CLASS => __PACKAGE__,
		TYPE => 'USER_QUIT',
		USER => $user,
		QUIT => $quit,
	};

	$get_quit_empty_chans->execute($id);

	$nick_chan_delete->execute($id);
	$user_quit->execute($nick);

	while(my ($cn) = $get_quit_empty_chans->fetchrow_array) {
		chanserv::channel_emptied({CHAN => $cn}); # FIXME
	}
}

our ($get_squit_lock, $get_squit_empty_chans, $squit_nickreg, $squit_users);

proc_init {
	$get_squit_lock = $dbh->prepare("LOCK TABLES chanuser WRITE, chanuser AS cu1 READ LOCAL, chanuser AS cu2 READ LOCAL, user WRITE, nickreg WRITE, nickid WRITE, chanban WRITE, chan WRITE, chanreg READ LOCAL");
	$get_squit_empty_chans = $dbh->prepare("SELECT cu2.chan, COUNT(*) AS c
		FROM user, chanuser AS cu1, chanuser AS cu2
		WHERE user.server=? AND cu1.nickid=user.id
		AND cu1.chan=cu2.chan AND cu1.joined=1 AND cu2.joined=1
		GROUP BY cu2.chan HAVING c=1 ORDER BY NULL");
	$squit_users = $dbh->prepare("UPDATE chanuser, user
		SET chanuser.joined=0, user.online=0, user.quittime=UNIX_TIMESTAMP()
		WHERE user.id=chanuser.nickid AND user.server=?");
	# Must call squit_nickreg first as squit_users modifies user.flags
	$squit_nickreg = $dbh->prepare("UPDATE nickreg, nickid, user
		SET nickreg.quit=?, nickreg.last=UNIX_TIMESTAMP()
		WHERE nickreg.nick=nickid.nick AND nickid.id=user.id
		AND user.online=1 AND user.server=?");
};

sub squit($$$) {
	my (undef, $server, $reason) = @_;

	$get_squit_lock->execute; $get_squit_lock->finish;

	$get_squit_empty_chans->execute($server);
	
	$squit_nickreg->execute("Netsplit from $server", $server);
	$squit_nickreg->finish;

	$squit_users->execute($server);
	$squit_users->finish;

	while(my ($cn) = $get_squit_empty_chans->fetchrow_array) {
		chanserv::channel_emptied({CHAN => $cn});
	}
	$get_squit_empty_chans->finish;

	$unlock_tables->execute; $unlock_tables->finish;
}

our ($nick_change);

proc_init {
	$nick_change = $dbh->prepare("UPDATE user SET nick=?, time=? WHERE nick=?");
};

sub nick_change($$$) {
	my ($old, $new, $time) = @_;

	return if(lc $old eq lc $new);

	nick_cede($new);
	
	$nick_change->execute($new, $time, $old);
	$add_nickchg->execute($ircline, $new, $new);
	
	ircd::setumode($nsnick, $new, '-r');

	if($new =~ /^guest/i) {
		$get_guest->execute($new);
		if($get_guest->fetchrow_array) {
			$set_guest->execute(0, $new);
		} else {
			guestnick($new);
		}
		return;
	}
	
	check_identify({ NICK => $new });
}

1;
