#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::User::Host;

use strict;

use Exporter 'import';
BEGIN { our @EXPORT_OK = qw(userip) }

use SrSv::MySQL '$dbh';
use SrSv::Process::Init;
use SrSv::IRCd::Event qw(addhandler);
use SrSv::Message qw(message);
use SrSv::User qw(get_user_id);
use SrSv::User::Event;

addhandler('CHGHOST', undef, undef, 'chghost');
addhandler('CHGIDENT', undef, undef, 'chgident');
addhandler('USERIP', undef, undef, 'userip');

our ($update_regnick_vhost, $set_ident, $set_vhost, $set_ip);

proc_init {
	$update_regnick_vhost = $dbh->prepare("UPDATE nickreg,nickid SET nickreg.vhost=?
		WHERE nickreg.id=nickid.nrid AND nickid.id=?");
	$set_ident = $dbh->prepare("UPDATE user SET ident=? WHERE id=?");
	$set_vhost = $dbh->prepare("UPDATE user SET vhost=? WHERE id=?");
	$set_ip = $dbh->prepare("UPDATE user SET ip=? WHERE id=?");
};

sub userip($$$) {
	my($src, $nick, $ip) = @_;
	my $user = { 'NICK' => $nick };
	
	#my $targetid = get_nick_id($target);
	my $iip; my @ips = split(/\./, $ip);
	for(my $i; $i < 4; $i++) {
		$iip += $ips[$i] * (2 ** ((3 - $i) * 8));
	}

	set_ip($user, $iip);

	message {
		CLASS => __PACKAGE__,
		TYPE => 'USERIP',
		USER => $user,
		IIP => $iip,
		ON_FINISH => 'SrSv::User::Event::nick_create_post',
		SYNC => 1,
	};
}

sub chghost($$$) {
	my ($src, $dst, $vhost) = @_;
	my $user = { NICK => $dst };
	my $uid = get_user_id($user);
	
	#my ($cur_ident, $cur_vhost) = get_vhost($targetid);
	#if $cur_vhost eq '*' {
	$update_regnick_vhost->execute($vhost, $uid);
	$update_regnick_vhost->finish();
	#}
	
	set_vhost($user, $vhost);
	chanserv::akick_alluser($user);
}

sub chgident($$$) {
	my ($src, $dst, $ident) = @_;
	my $user = { NICK => $dst };
	
	set_ident($user, $ident);
	chanserv::akick_alluser($user);
}

sub set_vhost($$) {
	my ($user, $vhost) = @_;
	my $id = get_user_id($user);
	
	return $set_vhost->execute($vhost, $id);
}

sub set_ident($$) {
	my ($user, $ident) = @_;
	my $id = get_user_id($user);
	
	return $set_ident->execute($ident, $id);
}

sub set_ip($$) {
	my ($user, $ip) = @_;
	my $id = get_user_id($user);

	return $set_ip->execute($ip, $id);
}

1;
