#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::User::Mode;

use strict;

use Exporter 'import';
BEGIN { our @EXPORT = qw(get_umodes) }

use SrSv::Process::Init;
use SrSv::MySQL '$dbh';
use SrSv::IRCd::Event qw(addhandler);
use SrSv::User qw(get_user_id);

addhandler('UMODE', undef, undef, 'umode');

our ($get_umodes, $set_umodes);

proc_init {
	$get_umodes = $dbh->prepare("SELECT modes FROM user WHERE id=?");
	$set_umodes = $dbh->prepare("UPDATE user SET modes=? WHERE id=?");
};

sub umode($$) {
	my ($nick, $modes) = @_;
	my $user = { NICK => $nick };

	my $id = get_user_id($user);
	
	$get_umodes->execute($id);
	my ($omodes) = $get_umodes->fetchrow_array;
	$set_umodes->execute(modes::add($omodes, $modes, 0), $id);

	#this could be optimized
	# if -x call chghost('', $nick, realhost)
	ircd::userhost($nick) if ($modes =~ m/x/);
}

sub get_umodes($) {
	my $uid = get_user_id(shift);

	$get_umodes->execute($uid);
	my ($modes) = $get_umodes->fetchrow_array;
	$get_umodes->finish;

	return $modes;
}

1;
