#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::Web::Login;

use strict;

use Exporter 'import';
BEGIN {
	our @EXPORT = qw(chk_login);
	our @EXPORT_OK = qw(get_digest);
}

use Digest::MD5 'md5_base64';

use SrSv::Web::Content;

use SrSv::NickReg qw(is_registered);
use SrSv::NickReg::Password qw(get_pass chk_pass);

sub get_digest($) {
	my ($nick) = @_;

	return md5_base64($nick . get_pass($nick));
}

sub chk_login($) {
	my ($cgi) = @_;

	if($cgi->param('pass')) { check_pass($cgi); }
	else { check_cookie($cgi); }
}

sub check_pass($) {
	my ($cgi) = @_;

	my $nick = $cgi->param('nick');

	do_login_form($cgi, qq{<p>The nick <b>$nick</b> is not registered.</p>}) unless is_registered($nick);
	
	if(chk_pass($nick, $cgi->param('pass'), undef)) {
		cookie $cgi->cookie (
			-name => 'NickAuth',
			-value => "$nick," . get_digest($nick),
			-path => '/',
		);

		return 1;
	} else {
		do_login_form($cgi, '<p>Password incorrect.</p>');
	}
}

sub check_cookie($) {
	my ($cgi) = @_;

	my $cookie = $cgi->cookie('NickAuth');

	do_login_form($cgi) unless $cookie;
	
	my ($nick, $digest) = split(/,/, $cookie);
	
	do_login_form($cgi) unless($digest eq get_digest($nick));

	return $nick;
}

sub do_login_form($;$) {
	my ($cgi, $message) = @_;
	$message = '<p>Please identify with NickServ.</p>' unless $message;
	
	my $action = $cgi->url(-relative=>1);

	body qq{<h2>Identification required</h2>$message
<form action="$action" method="post">
<table class="entrytable">
<tr>
<td><label for="nick">NickServ Nick:</label></td>
<td><input type="text" id="nick" name="nick"></td>
</tr>
<tr>
<td><label for="pass">Password:</label></td>
<td><input type="password" id="pass" name="pass"></td>
</tr>
<tr>
<td colspan="2" style="text-align: center;">
 <input type="submit" value="Continue" style="width: 9em; height: 2.3em;">
</td>
</tr>
</table>
</form>};

	print_content($cgi);

	exit;
}

1;
