#!/usr/bin/perl

#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
use strict;
use DBI;

use libs::config;
use modules::serviceslibs::svsdb;

my %conf = config::readHash("config/services.conf");

print "Connecting...\n";

my $dbh;
eval { $dbh = DBI->connect("DBI:mysql:".$conf{'mysql-db'}, $conf{'mysql-user'}, $conf{'mysql-pass'}, {  AutoCommit => 1, RaiseError => 1 }) };

if($@) {
	print "FATAL: Can't connect to database:\n$@\n";
	print "You must edit config/services.conf and create a corresponding\nMySQL user and database!\n\n";
	exit;
}

print "Updating schema...\n";

$dbh->{RaiseError} = 0;
$dbh->{PrintError} = 0;

for (qw(createdb.sql updatedb.sql)) {
	open SQL, $_;
	my $sql;
	while(my $x = <SQL>) { $sql .= $x unless $x =~ /^#/ or $x eq '\n'}
	foreach my $line (split(/;/s, $sql)) {
		$dbh->do($line);
	}
}

print "Updating chanperm...\n";

my $add_perm = $dbh->prepare("INSERT IGNORE INTO chanperm SET name=?, level=?, max=?");
my $del_perm = $dbh->prepare("DELETE FROM chanperm WHERE name=?");

my $get_max = $dbh->prepare("SELECT max FROM chanperm WHERE name=?");
my $set_max = $dbh->prepare("UPDATE chanperm SET max=? WHERE name=?");

my @perms = (
#	Name		Default	Max
        ['Join',	0,	],
        ['AccList',	1,	],
        ['AccChange',	5,	],
        ['AKICK',	5,	],
	['AKickList',	3,	],
        ['SET',		6,	],
        ['BAN',		4,	],
        ['CLEAR',	6,	],
        ['GETKEY',	4,	],
        ['INFO',	0,	],
        ['KICK',	4,	],
	['LEVELS',	6,	7],
	['LevelsList',	3,	7],
        ['INVITE',	4,	],
	['InviteSelf',	1,	],
        ['TOPIC',	5,	],
        ['UNBAN',	4,	],
        ['VOICE',	2,	],
        ['HALFOP',	3,	],
        ['OP',		4,	],
        ['ADMIN',	5,	],
        ['OWNER',	6,	],
        ['Memo',	5,	],
        ['BadWords',	5,	],
        ['Greet',	8,	],
        ['NoKick',	4,	],
        ['BotSay',	5,	],
        ['BotAssign',	6,	],
        ['SetTopic',	0,	],
	['WELCOME',	6,	],
);

my @noperms = ();

foreach my $p (@perms) {
	my ($name, $default, $new_max) = @$p;

	$new_max = 8 unless defined($new_max);
	
	$add_perm->execute($name, $default, $new_max);

	$get_max->execute($name);
	my $old_max = $get_max->fetchrow_array;

	if($old_max == 0) {
		$set_max->execute($new_max, $name);
	}
}

foreach my $p (@noperms) {
	$del_perm->execute($p);
}

print "Updating nick IDs...\n";

$svsdb::dbh = $dbh;
svsdb::nrid_convert();

print "Database setup complete!\n";
