#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package ircd;

use strict;

use IO::Socket::INET;
use Event;
use Carp;
use MIME::Base64;

use SrSv::Conf 'main';

use SrSv::Debug;

use SrSv::IRCd::IO qw(ircd_connect ircsend ircsendimm ircd_flush_queue);
use SrSv::IRCd::Event qw(addhandler callfuncs);
use SrSv::IRCd::State qw($ircline $remoteserv $ircd_ready synced initial_synced set_server_state get_server_state get_online_servers);
use SrSv::IRCd::ValidNick qw($valid_nick_re);

use SrSv::Unreal::Modes qw(@opmodes %opmodes $scm $ocm $acm);
use SrSv::Unreal::Tokens;

use SrSv::Agent;

# FIXME
use constant {
	# Wait For
	WF_NONE => 0,
	WF_NICK => 1,
	WF_CHAN => 2,
	WF_ALL => 3,
};

our $sjoin = 1;
our $noquit = 1;
our $nickip = 1;
our $sjb64 = 1;

our %defer_mode;

addhandler('POSTSEOS', undef(), undef(), 'ircd::eos', 1);
addhandler('NETINFO', undef(), undef(), 'ircd::netinfo', 1);
addhandler('VERSION', undef(), undef(), 'ircd::version', 1);

sub init() {
	my $remote = $main_conf{remote};
	my $port = $main_conf{port};

	ircd_connect($remote, $port);

	ircsendimm('PROTOCTL '.($tkn ? 'TOKEN ' : '').'NICKv2 UMODE2 TKLEXT VHP'.
		($noquit ? ' NOQUIT' : '').($sjoin ? ' SJOIN SJOIN2 SJ3' : '').
		($nickip ? ' NICKIP' : '').
		($sjb64 ? ' SJB64 NS' : ''),
		'PASS :'.$main_conf{pass},
		'SERVER '.$main_conf{local}.' 0 '.$main_conf{numeric}.' :'.$main_conf{info});
}

# Handler functions

sub pong($;$$) {
        my ($src, $cookie, $dst) = @_;
	# This will only make sense if you remember that
	# $src is where it came from, $dst is where it went (us)
	# we're basically bouncing it back, but changing from PING to PONG.
	if (defined($dst) and defined($cookie)) {
		# $dst is always $main_conf{local} anyway...
		# this is only valid b/c we never have messages routed THROUGH us
		# we are always an end point.
		ircsendimm(":$dst $tkn{PONG}[$tkn] $src :$cookie");
	}
	else {
		ircsendimm("$tkn{PONG}[$tkn] :$src");
        }
}

sub eos {
	print "GOT EOS\n\n" if DEBUG();

	#foreach my $k (keys %servers) {
	#	print "Server: $k ircline: ",$servers{$k}[0], " state: ", $servers{$k}[1], "\n";
	#}
	#print "Synced: ", synced(), "\n\n";
	#exit;
	
	ircsendimm(':'.$main_conf{local}.' '.$tkn{EOS}[$tkn]);
	#$ready = 1;

	agent_sync();
	ircd_flush_queue();
	
	$ircd_ready = 1;
}

sub netinfo($$$$$$$) {
	my $message = shift;
	ircsendimm($tkn{NETINFO}[$tkn].' 0 '.time." $_[1] $_[2] 0 0 0 :$_[6]");
	$main_conf{network} = $_[6];
}

sub tssync {
	ircsendimm(":$main_conf{local} $tkn{TSCTL}[$tkn] SVSTIME ".time);
}

# Send Functions

sub kick($$$$) {
	my ($src, $chan, $target, $reason) = @_;
	ircsend(":$src $tkn{KICK}[$tkn] $chan $target :$reason");
#	thread::ircrecv(":$src $tkn{KICK}[$tkn] $chan $target :$reason");
	callfuncs('KICK', 0, 2, [$src, $chan, $target, $reason]);
}

sub invite($$$) {
	my ($src, $chan, $target) = @_;
	#:SecurityBot INVITE tabris #channel
	ircsend(":$src $tkn{INVITE}[$tkn] $target $chan");
}

sub ping {
#	if(@_ == 1) {
		ircsend(':'.$main_conf{local}.' '.$tkn{PING}[$tkn].' :'.$main_conf{local});
#	} else {
#		ircsend(':'.$_[2].' '.$tkn{PONG}[$tkn].' '.$_[0].' :'.$_[1]);
#	}
}

sub privmsg($$@) {
	my ($src, $dst, @msgs) = @_;

	my @bufs;
	foreach my $buf (@msgs) {
		# 3 spaces, two colons, PRIVMSG=7
		my $buflen = length($src) + length($dst) + 12;
		push @bufs, fmt::wordwrap($buf, (511 - 75 - $buflen));
	}

	# submit a list of messages as a single packet to the server
	ircsend(":$src $tkn{PRIVMSG}[$tkn] $dst :".join("\r\n".":$src $tkn{PRIVMSG}[$tkn] $dst :", @bufs));
}

sub debug(@) {
	my (@msgs) = @_;
	debug_privmsg($config::conf{local}, $config::conf{diag}, @msgs);
	#log::write_log('diag', '<'.$config::conf{local}.'>', @msgs);
}

sub debug_privmsg($$@) {
	my ($src, $dst, @msgs) = @_;

	my @bufs;
	foreach my $buf (@msgs) {
		# 3 spaces, two colons, PRIVMSG=7
		my $buflen = length($src) + length($dst) + 12;
		push @bufs, fmt::wordwrap($buf, (511 - 75 - $buflen));
	}

	# submit a list of messages as a single packet to the server
	ircsendimm(":$src $tkn{PRIVMSG}[$tkn] $dst :".join("\r\n".":$src $tkn{PRIVMSG}[$tkn] $dst :", @bufs));
}

sub notice($$@) {
	my ($src, $dst, @msgs) = @_;

	my @bufs;
	foreach my $buf (@msgs) {
		# 3 spaces, two colons, PRIVMSG=7
		my $buflen = length($src) + length($dst) + 12;
		push @bufs, fmt::wordwrap($buf, (511 - 75 - $buflen));
	}

	# submit a list of notices as a single packet to the server
	ircsend(":$src $tkn{NOTICE}[$tkn] $dst :".join("\r\n".":$src $tkn{NOTICE}[$tkn] $dst :", @bufs));
}

sub ctcp($$@) {
	my ($src, $dst, $cmd, @toks) = @_;

	privmsg($src, $dst, "\x01".join(' ', ($cmd, @toks))."\x01");
}

sub ctcp_reply($$@) {
	my ($src, $dst, $cmd, @toks) = @_;

	notice($src, $dst, "\x01".join(' ', ($cmd, @toks))."\x01");
}

sub setumode($$$) {
	my ($src, $dst, $modes) = @_;

	ircsend(":$src $tkn{SVS2MODE}[$tkn] $dst $modes");
	callfuncs('UMODE', 0, undef, [$dst, $modes]);
}

sub setmode($$$;$) {
	my ($src, $dst, $modes, $parms) = @_;

	callfuncs('MODE', undef, 1, [$src, $dst, $modes, $parms]);
	
	print "$ircline -- setmode($src, $dst, $modes, $parms)\n" if DEBUG();
	my $prev = $defer_mode{"$src $dst"}[-1];

	if(defined($prev)) {
		my ($oldmodes, $oldparms) = split(/ /, $prev, 2);
		
		# 12 modes per line
		if((length($oldmodes.$modes) - @{[($oldmodes.$modes) =~ /[+-]/g]}) <= 12 and length($src.$dst.$parms.$oldparms) < 400) {
			$defer_mode{"$src $dst"}[-1] = modes::merge(
				$prev, "$modes $parms", ($dst =~ /^#/ ? 1 : 0));
			print $defer_mode{"$src $dst"}[-1], " *** \n" if DEBUG();
			
			return;
		}
	}
	
	push @{$defer_mode{"$src $dst"}}, "$modes $parms";
}

sub flushmodes {
	my @k = keys(%defer_mode); my @v = values(%defer_mode);
	
	for(my $i; $i<@k; $i++) {
		my ($src, $dst) = split(/ /, $k[$i]);
		my @m = @{$v[$i]};
		foreach my $m (@m) {
			my ($modes, $parms) = split(/ /, $m, 2);

			setmode_real($src, $dst, $modes, $parms);
		}
	}

	%defer_mode = ();
}

sub setmode_real($$$;$) {
	my ($src, $dst, $modes, $parms) = @_;

	print "$ircline -- setmode_real($src, $dst, $modes, $parms)\n" if DEBUG();
	ircsend(":$src ".$tkn{MODE}[$tkn]." $dst $modes".($parms?" $parms":''));
}

sub settopic($$$$$) {
	my ($src, $chan, $setter, $time, $topic) = @_;
	
	ircsend(":$src ".$tkn{TOPIC}[$tkn]." $chan $setter $time :$topic");
	callfuncs('TOPIC', undef, undef, [$src, $chan, $setter, $time, $topic]);
}

sub wallops ($$) {
	my ($src, $message) = @_;
	ircsend(":$src $tkn{WALLOPS}[$tkn] :$message");
}

sub globops ($$) {
	my ($src, $message) = @_;
	ircsend(":$src $tkn{GLOBOPS}[$tkn] :$message");
}

sub kline ($$$$$) {
        my ($setter, $ident, $host, $expiry, $reason) = @_;
	$setter=$main_conf{local} unless defined($setter);
	$ident = '*' unless defined($ident);


	#foreach my $ex (@except) { return 1 if $mask =~ /\Q$ex\E/i; }
	
	#my $line = "GLINE $mask $time :$reason";
	# you need to use TKL for this. GLINE is a user command
	# TKL is a server command.	
        # format is
        # TKL +/- type ident host expiretime timenow :reason
#:nascent.surrealchat.net TKL + G * *.testing.only tabris!northman@netadmin.SCnet.ops 1089168439 1089168434 :This is just a test.
        my $line = "TKL + G $ident $host $setter ".($expiry + time()).' '.time()." :$reason";

	#notice($nick, $conf{"Debugger"}, $line) if $conf{"Debug"};
	ircsend($line);
}

sub unkline ($$$) {
	my ($setter, $ident, $host) = @_;
# TKL - G ident *.test.dom tabris!northman@netadmin.SCnet.ops
	ircsend("TKL - G $ident $host $setter");
}

sub zline ($$$$) {
        my ($setter, $host, $expiry, $reason) = @_;
	$setter=$main_conf{local} unless defined($setter);

	#foreach my $ex (@except) { return 1 if $mask =~ /\Q$ex\E/i; }
	
	#my $line = "GZLINE $mask $time :$reason";
	# you need to use TKL for this. GLINE is a user command
	# TKL is a server command.	
        # format is
        # TKL +/- type ident host expiretime timenow :reason
        my $line = "TKL + Z * $host $setter ".($expiry + time).' '.time." :$reason";
	#notice($nick, $conf{"Debugger"}, $line) if $conf{"Debug"};
	ircsend($line);
}

sub unzline ($$$) {
	my ($setter, $ident, $host) = @_;
# TKL - G ident *.test.dom tabris!northman@netadmin.SCnet.ops
	ircsend("TKL - Z $ident $host $setter");
}

sub irckill($$$) {
	my ($src, $target, $reason) = @_;
	#FIXME: this needs to be checked for correctness
	ircsendimm(":$src ".$tkn{KILL}[$tkn]." $target :$src ($reason)");
	callfuncs('KILL', 0, 1, [$src, $target, $src, $reason]);
}

sub svssno($$$) {
    my ($src, $target, $snomasks) = @_;
    $src=$main_conf{local} unless defined($src);
    # TODO:
    # None, this doesn't affect us.

    # SVSSNO is not in tokens.txt nor msg.h
    # however there is SENDSNO. no clue which to use.
    # SENDSNO may be a command by which snomasks are 
    # passed from server to server.
    # not observed in the wild yet.
    ircsend(":$src ".'SVS2SNO'." $target $snomasks ".time);
}

sub svsnick($$$) {
    my ($src, $oldnick, $newnick) = @_;
    $src=$main_conf{local} unless defined($src);
    # note: we will get a NICK cmd back after a 
    # successful nick change.
    # warning, if misused, this can KILL the user
    # with a collision
    
#    ircsend(":$src ".$tkn{SVSNICK}[$tkn]." $oldnick $newnick ".time);
    ircsend($tkn{SVSNICK}[$tkn]." $oldnick $newnick :".time);
}

sub svsnoop($$$) {
    my ($targetserver, $bool, $src) = @_;
    $src = $main_conf{local} unless defined($src);
    if ($bool > 0) { $bool = '+'; } else { $bool = '-'; }
#this is SVS NO-OP not SVS SNOOP
    ircsend(":$main_conf{local} $tkn{SVSNOOP}[$tkn] $targetserver $bool");
}

sub svswatch ($$@) {
# Changes the WATCH list of a user.
# Syntax: SVSWATCH <nick> :<watch parameters>
# Example: SVSWATCH Blah :+Blih!*@* -Bluh!*@* +Bleh!*@*.com
# *** We do not track this info nor care.
    my ($src, $target, @watchlist) = @_;
    # SVSWATCH is not in msg.h
    ircsend(":$src SVSWATCH $target :".join(' ', @watchlist));
}

sub svssilence ($$@) {
# Changes the SILENCE list of a user.
# Syntax: SVSSILENCE <nick> :<silence parameters>
# Example: SVSSILENCE Blah :+Blih!*@* -Bluh!*@* +Bleh!*@*.com
# *** We do not track this info nor care.
    my ($src, $target, @silencelist) = @_;
    # SVSSILENCE is not in msg.h
    ircsend(":$src SVSSILENCE $target :".join(' ', @silencelist));
}

sub svso($$$) {
# Gives nick Operflags like the ones in O:lines.
# SVSO <nick> <+operflags> (Adds the Operflags)
# SVSO <nick> - (Removes all O:Line flags)
# Example: SVSO SomeNick +bBkK
# *** We do not track this info nor care.
# *** We will see any umode changes later.
# *** this cmd does not change any umodes!

    my ($src, $target, $oflags) = @_;
    $src = $main_conf{local} unless defined($src);
    ircsend(":$src $tkn{SVSO}[$tkn] $target $oflags");

}

sub swhois($$$) {
# *** We do not track this info nor care.
    my ($src, $target, $swhois) = @_;
    $src = $main_conf{local} unless defined($src);
    ircsend(":$src $tkn{SWHOIS}[$tkn] $target :$swhois");
}

sub svsjoin($$@) {
    my ($src, $target, @chans) = @_;
    # a note. a JOIN is returned back to us on success
    # so no need to process this command.
    # similar for svspart.
    ircsend(($src?":$src":'')." $tkn{SVSJOIN}[$tkn] $target ".join(',', @chans));
}

sub svspart($$$@) {
    my ($src, $target, $reason, @chans) = @_;
    ircsend(($src ? ":$src" : '')." $tkn{SVSPART}[$tkn] $target ".join(',', @chans).
    	($reason ? " :$reason" : ''));
}

sub sqline ($;$) {
# possibly implement with TKL Q?
# we need to sqline most/all of our agents.
# tho whether we want to put it in agent_connect
# or leave it to the module to call it...
    my ($nickmask, $reason) = @_;
    ircsend("$tkn{SQLINE}[$tkn] $nickmask".($reason?" :$reason":''));
}

sub qline($$$) {
	my ($nickmask, $expiry, $reason) = @_;
# TKL version - Allows timed qlines.
# TKL + Q * test services.SC.net 0 1092179497 :test
    ircsend('TKL + Q * '.$nickmask.' '.$main_conf{local}.' '.($expiry ? ($expiry + time()) : 0).' '.time().' '.$reason);
}

sub unsqline ($) {
# possibly implement with TKL Q?
# we need to sqline most/all of our agents.
# tho whether we want to put it in agent_connect
# or leave it to the module to call it...
    my ($nickmask) = @_;
    ircsend($tkn{UNSQLINE}[$tkn].' '.$nickmask);
}

sub unqline($) {
	my ($nickmask) = @_;
# TKL version
# TKL - Q * test services.SC.net

    ircsend('TKL - Q * '.$nickmask.' '.$main_conf{local});
}

sub svskill($$$) {
	my ($src, $target, $reason) = @_;
	# SVSKILL requires a src, it will NOT work w/o one.
	# not sure if it'll accept a servername or not.
	# consider defaulting to ServServ
	die('svskill called w/o $src') unless $src;
	ircsend(':'.$src.' '.$tkn{SVSKILL}[$tkn].' '.$target.' :'.$reason);
	callfuncs('QUIT', 0, undef, [$target, $reason]);
}

sub version($) {
	my ($src) = @_;
	ircsend(":$main_conf{local} 351 $src $main::progname ver $main::version $main_conf{local} ".
		$main::extraversion);
}

sub userhost($) {
	my ($target) = @_;
	ircsend($tkn{USERHOST}[$tkn]." $target");
}

sub userip($) {
	my ($target) = @_;
	die "We're not supposed to use USERIP anymore!" if DEBUG_ANY and $nickip;
	ircsend(":$main::rsnick USERIP $target");
}

sub chghost($$$) {
	my ($src, $target, $vhost) = @_;
	ircsend(($src?":$src ":'').$tkn{CHGHOST}[$tkn]." $target $vhost");
        callfuncs('CHGHOST', 0, 1, [$src, $target, $vhost]);
}

sub chgident($$$) {
	my ($src, $target, $ident) = @_;
	ircsend(($src?":$src ":'').$tkn{CHGIDENT}[$tkn]." $target $ident");
        callfuncs('CHGIDENT', 0, 1, [$src, $target, $ident]);
}

sub jupe_server($$) {
	my ($server, $reason) = @_;

	# :nascent.surrealchat.net SERVER wyvern.surrealchat.net 2 :SurrealChat
	ircsend(':'.$main_conf{local}.' '.$tkn{SERVER}[$tkn]." $server 2 :$reason");

	# We don't need a callback as long as we don't care about spurious SQUITs
	# Consider set_server_state($server, -1) for a special value for jupes
}

sub rehash_all_servers(;$) {
	my ($type) = @_;

	# Validate the type before passing it along.
	# Very IRCd specific! May be version specific.
	$type = undef() if(defined($type) && !($type =~ /^\-(motd|botmotd|opermotd|garbage)$/i));

	foreach my $server (get_online_servers()) {
		ircsend(':'.$main::rsnick.' '.$tkn{REHASH}[$tkn].' '.$server.(defined($type) ? ' '.$type : '') );
	}
}

1;
