#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package nickserv;

#require Exporter;
#@ISA = qw(Exporter);
#@EXPORT_OK = qw(get_user_id get_user_nick is_identified notice);

use strict;
use threads::shared;
use Time::Local;

use SrSv::Timer qw(add_timer);

use SrSv::Process::Init;
use SrSv::MySQL '$dbh';

use SrSv::Agent;
use SrSv::Conf qw(main services);
use SrSv::Errors;

use SrSv::IRCd::State qw(synced);

use SrSv::ParseMask;

use SrSv::NickServ;
#use SrSv::NickServ::Agent '/./';
# FIXME
use SrSv::NickServ::UI qw(ns_dispatch);
*dispatch = \&ns_dispatch;

use SrSv::NickServ::Id '/./';

# TODO: Make these optional
use SrSv::NickServ::Enforcer '/./';
use SrSv::NickServ::Register;
use SrSv::NickServ::Settings;
use SrSv::NickServ::Settings::Boolean;
use SrSv::NickServ::Settings::Oper;
use SrSv::NickServ::Settings::Vacation;
use SrSv::NickServ::Greet;
use SrSv::NickServ::Info;
use SrSv::NickServ::SendPass;
use SrSv::NickServ::GetPass;
use SrSv::NickServ::Glist;
use SrSv::NickServ::Link;
use SrSv::NickServ::Alist;

use SrSv::NickReg;
use SrSv::NickReg::Flags;
use SrSv::NickReg::Id '/./';
use SrSv::NickReg::Seen 'do_seen';
use SrSv::NickReg::Password '/./';
use SrSv::NickReg::IRC '/./';
use SrSv::NickReg::Alias '/./';
use SrSv::NickReg::Protect '/./';
use SrSv::NickReg::Event;

use SrSv::User qw(:flags get_user_nick get_user_id);
use SrSv::User::Notice;
use SrSv::User::Mode '/./';

use SrSv::Errors;

use SrSv::Email::Validate '/./';

use SrSv::MySQL::Unlock;

use constant {


	#current nickreg.flags definition limits us to 16 of these. or 32768 as last flag

	OLD_USER_AGE => 300,
};

our $user_table : shared;

my $cur_lock;
my $cnt_lock = 0;

our (
	$nick_check, $nick_create, $nick_create_old, $nick_change, $nick_quit, $nick_delete, $nick_id_delete, $get_quit_empty_chans, $nick_chan_delete,
	$get_hostless_nicks,

	$get_squit_lock, $squit_users, $squit_nickreg, $get_squit_empty_chans,

	$get_nickchg, $del_nickchg_id, $add_nickchg, $reap_nickchg,
	
	$get_user_id, $get_user_nick,
	$get_nick_inval, $inc_nick_inval,
	$get_id_nicks, $get_using_nicks, $is_identified, $is_registered,
	$is_alias_of,

	$get_guest, $set_guest,

	$get_lock, $release_lock,
	
	$get_umodes, $set_umodes,
	
	$get_info, $get_host, $get_vhost, $set_vhost, $set_ident, $set_ip, $get_user_info,
	$update_regnick_vhost, $get_regd_time,

	$chk_clone_except, $count_clones,
	
	$get_pass, $get_pass_email, 
	$set_pass, $set_email,
	
	$set_flags, $get_flags, $set_flag, $unset_flag,

	$get_user_flags, $set_user_flag, $unset_user_flag, $set_user_flag_all,

	$get_root_nick, $chk_pass, $identify, $identify_ign, $id_update, $logout, $unidentify, $unidentify_single,
	$update_lastseen, $quit_update,

	$get_register_lock, $register, $create_alias, $drop, $change_root,

	$count_aliases, $get_random_alias, $delete_alias, $delete_aliases, $get_all_access, $del_all_access,
	$change_all_access, $change_akicks, $change_founders, $change_successors, $change_svsops,
	
	$lock_user_table, $cleanup, $unlock_tables,
	$is_online,

	$get_matching_nicks,

	$cleanup_nickid, $cleanup_users, $get_expired, $get_near_expired, $set_near_expired,
	
	$get_watches, $check_watch, $set_watch, $del_watch, $chgroot_watch,
	$get_silences, $check_silence, $set_silence, $del_silence, $chgroot_silence,

	$get_seen
);

proc_init {



	$get_hostless_nicks = $dbh->prepare("SELECT nick FROM user WHERE vhost='*'");






	$reap_nickchg = $dbh->prepare("DELETE FROM nickchg WHERE seq<?");


	

	$get_lock = $dbh->prepare("SELECT GET_LOCK(?, 10)");
	$release_lock = $dbh->prepare("SELECT RELEASE_LOCK(?)");

	
	
	$get_host = $dbh->prepare("SELECT ident, host FROM user WHERE id=?");
	$get_vhost = $dbh->prepare("SELECT ident, vhost FROM user WHERE id=?");

	$get_user_info  = $dbh->prepare("SELECT ident, host, vhost, gecos, server FROM user WHERE id=?");
	





	



	




	
	$set_flags = $dbh->prepare("UPDATE nickreg, nickalias SET nickreg.flags=? WHERE nickalias.nrid=nickreg.id AND nickalias.alias=?");
	

	$get_user_flags = $dbh->prepare("SELECT flags FROM user WHERE id=?");
	$set_user_flag = $dbh->prepare("UPDATE user SET flags=IF(flags & ?, flags, flags ^ ?) WHERE id=?");
	$unset_user_flag = $dbh->prepare("UPDATE user SET flags=IF(flags & ?, flags ^ ?, flags) WHERE id=?");
	$set_user_flag_all = $dbh->prepare("UPDATE user SET flags=flags | ?");


	


	




	
	
	

	$cleanup = $dbh->prepare("DELETE FROM nickid USING nickid LEFT JOIN user ON (nickid.id=user.id) WHERE user.id IS NULL");
	



	$get_matching_nicks = $dbh->prepare("SELECT nickalias.alias, nickreg.nick, nickreg.ident, nickreg.vhost FROM nickalias, nickreg WHERE nickalias.nrid=nickreg.id AND nickalias.alias LIKE ? AND nickreg.ident LIKE ? AND nickreg.vhost LIKE ? LIMIT 50");
	
	$cleanup_nickid = $dbh->prepare("DELETE FROM nickid, user USING nickid
		LEFT JOIN user ON(nickid.id=user.id)
		WHERE user.id IS NULL OR (!(user.flags & ".UF_ONLINE.") AND quittime<?)");
	$cleanup_users = $dbh->prepare("DELETE FROM user WHERE !(flags & ".UF_ONLINE.") AND quittime<?");
	
	$get_expired = $dbh->prepare("SELECT nickreg.nick, nickreg.email, nickreg.ident, nickreg.vhost
		FROM nickreg LEFT JOIN nickid ON(nickreg.id=nickid.nrid)
		LEFT JOIN svsop ON(nickreg.id=svsop.nrid)
		WHERE nickid.nrid IS NULL AND svsop.nrid IS NULL AND nickreg.nearexp!=0 AND
		( ( !(nickreg.flags & " . NRF_HOLD . ") AND !(nickreg.flags & " . NRF_VACATION . ") AND nickreg.last<? ) OR
		( (nickreg.flags & " . NRF_VACATION . ") AND nickreg.last<? )
		)");
	$get_near_expired = $dbh->prepare("SELECT nickreg.nick, nickreg.email
		FROM nickreg LEFT JOIN nickid ON(nickreg.id=nickid.nrid) 
		LEFT JOIN svsop ON(nickreg.id=svsop.nrid)
		WHERE nickid.nrid IS NULL AND svsop.nrid IS NULL AND nickreg.nearexp=0 AND
		( ( !(nickreg.flags & " . NRF_HOLD . ") AND !(nickreg.flags & " . NRF_VACATION . ") AND nickreg.last<? ) OR
		( (nickreg.flags & " . NRF_VACATION . ") AND nickreg.last<? )
		)");
	$set_near_expired = $dbh->prepare("UPDATE nickreg SET nearexp=1, last=IF(? > last+?, ?, last) WHERE nick=?");
	
	$get_watches = $dbh->prepare("SELECT watch.mask, watch.time FROM watch, nickreg WHERE watch.nrid=nickreg.id AND nickreg.nick=?");
	$check_watch = $dbh->prepare("SELECT 1 FROM watch, nickreg WHERE watch.nrid=nickreg.id AND nickreg.nick=? AND watch.mask=?");
	$set_watch = $dbh->prepare("INSERT INTO watch SELECT id, ?, ? FROM nickreg WHERE nick=?");
	$del_watch = $dbh->prepare("DELETE FROM watch USING watch, nickreg WHERE watch.nrid=nickreg.id AND nickreg.nick=? AND watch.mask=?");

	$get_silences = $dbh->prepare("SELECT silence.mask, silence.time, silence.expiry FROM silence, nickreg WHERE silence.nrid=nickreg.id AND nickreg.nick=?");
	$check_silence = $dbh->prepare("SELECT 1 FROM silence, nickreg WHERE silence.nrid=nickreg.id AND nickreg.nick=? AND silence.mask=?");
	$set_silence = $dbh->prepare("INSERT INTO silence SELECT id, ?, ?, ? FROM nickreg WHERE nick=?");
	$del_silence = $dbh->prepare("DELETE FROM silence USING silence, nickreg WHERE silence.nrid=nickreg.id AND nickreg.nick=? AND silence.mask=?");

};

### NICKSERV COMMANDS ###

sub ns_list($$) {
	my ($user, $mask) = @_;

	unless(adminserv::is_svsop($user, adminserv::S_HELP())) {
		notice($user, $err_deny);
		return;
	}

	my ($mnick, $mident, $mhost) = parse_mask($mask);
	
	$mnick = '%' if($mnick eq '');
	$mident = '%' if($mident eq '');
	$mhost = '%' if($mhost eq '');

	my @data;
	$get_matching_nicks->execute($mnick, $mident, $mhost);
	while(my ($rnick, $rroot, $rident, $rhost) = $get_matching_nicks->fetchrow_array) {
		push @data, [$rnick, ($rroot ne $rnick ? $rroot : ''), $rident . '@' . $rhost];
	}

	notice($user, 
		"Registered nicks matching \002$mask\002:",
		fmt::columnar(@data)
	);
}

sub ns_watch($$$;$) {
	my ($user, $target, $cmd, $mask) = @_;
	my $src = get_user_nick($user);
	
	my $root = get_root_nick($target);
	unless ($root) {
		notice($user, "\002$target\002 is not registered.");
		return;
	}
	unless(is_identified($user, $target)) {
		notice($user, $err_deny);
		return;
	}
	
	if ($cmd =~ /^add$/i) {
		if($mask =~ /\!/ or $mask =~ /\@/) {
			my ($mnick, $mident, $mhost) = parse_mask($mask);
			if ($mnick =~ /\*/) {
				notice($user, "Invalid mask: \002$mask\002", 
					'A WATCH mask cannot wildcard the nick.');
				return;
			}
		}

		$check_watch->execute($root, $mask);
		if ($check_watch->fetchrow_array) {
			notice($user, "\002$mask\002 is already in \002$target\002's watch list.");
			return;
		}

		$set_watch->execute($mask, time(), $root);
		ircd::svswatch($nsnick, $src, "+$mask");
		notice($user, "\002$mask\002 added to \002$target\002's watch list.");
		return;
	}
	elsif ($cmd =~ /^del(ete)?$/i) {
		if($mask =~ /\!/ or $mask =~ /\@/) {
			my ($mnick, $mident, $mhost) = parse_mask($mask);
			if ($mnick =~ /\*/) {
				notice($user, "Invalid mask: \002$mask\002", 
					'A WATCH mask cannot wildcard the nick.');
			}
		}

		$check_watch->execute($root, $mask);
		unless ($check_watch->fetchrow_array) {
			notice($user, "\002$mask\002 is not in \002$target\002's watch list.");
			return;
		}
		$del_watch->execute($root, $mask);
		ircd::svswatch($nsnick, $src, "-$mask");
		notice($user, "\002$mask\002 removed from \002$target\002's watch list.");
	}
	elsif ($cmd =~ /^list$/i) {
		my @data;
		
		$get_watches->execute($root);
		while(my ($mask, $time) = $get_watches->fetchrow_array) {
			push @data, [$mask, scalar(localtime($time))];
		}
		
		notice($user,
			"Watch list for \002$target\002:",
			fmt::columnar(@data)
		);
	}
	else {
		notice($user, 'Syntax: WATCH <ADD|DEL|LIST> [nick]');
	}
}

sub ns_silence($$$;$$) {
	my ($user, $target, $cmd, $mask, $expiry) = @_;
	my $src = get_user_nick($user);
	
	my $root = get_root_nick($target);
	unless ($root) {
		notice($user, "\002$target\002 is not registered.");
		return;
	}
	
	unless(is_identified($user, $target)) {
		notice($user, $err_deny);
		return;
	}
	
	if ($cmd =~ /^add$/i) {
		unless($mask =~ /\!/ && $mask =~ /\@/) {
			notice($user, "Invalid mask: \002$mask\002",
				"A SILENCE mask must be in the format nick!ident\@host");
			return;
		}

		$check_silence->execute($root, $mask);
		if ($check_silence->fetchrow_array) {
			notice($user, "\002$mask\002 is already in \002$target\002's SILENCE list.");
			return;
		}
		
		if($expiry) {
			$expiry=misc::parse_time($expiry)+time();
		}
		else {
			$expiry = 0;
		};
		$set_silence->execute($mask, time(), $expiry, $root);
		ircd::svssilence($nsnick, $src, "+$mask");
		notice($user, "\002$mask\002 added to \002$target\002's SILENCE list.");
	}
	elsif ($cmd =~ /^del(ete)?$/i) {
		unless($mask =~ /\!/ && $mask =~ /\@/) {
			notice($user, "Invalid mask: \002$mask\002", 
				"A SILENCE mask must be in the format nick!ident\@host");
			return;
		}

		$check_silence->execute($root, $mask);
		unless ($check_silence->fetchrow_array) {
			notice($user, "\002$mask\002 is not in \002$target\002's watch list.");
			return;
		}
		$del_silence->execute($root, $mask);
		ircd::svssilence($nsnick, $src, "-$mask");
		notice($user, "\002$mask\002 removed from \002$target\002's SILENCE list.");
	}
	elsif ($cmd =~ /^list$/i) {
		$get_silences->execute($root);
		
		my @data;
		while(my ($mask, $time, $expiry) = $get_silences->fetchrow_array) {
			push @data, [$mask, scalar(localtime($time)),
				($expiry ? "Expires in ".misc::time_rel($expiry-time()) : "Never expires")
			];
		}
		
		notice($user,
			"Silence list for \002$target\002:",
			fmt::columnar(@data)
		);
	}
	else {
		notice($user, 'Syntax: SILENCE <ADD|DEL|LIST> [nick]');
	}
}

sub ns_acc($@) {
	my ($user, @targets) = @_;
	my @reply;

	foreach my $target (@targets) {
		unless(is_registered($target)) {
			push @reply, "$target 0", "\002$target\002 is not registered.";
			next;
		}

		unless(is_online($target)) {
			push @reply, "$target 1", "\002$target\002 is registered and offline.";
			next;
		}

		unless(is_identified({NICK => $target}, $target)) {
			push @reply, "$target 2", "\002$target\002 is online but not identified.";
			next;
		}

		push @reply, "$target 3", "\002$target\002 is registered and identified.";
	}
	notice($user, @reply);
}


### MISCELLANEA ###





sub kill_user($$) {
	my ($user, $reason) = @_;

	ircd::irckill(get_user_agent($user) || $main_conf{local}, get_user_nick($user), $reason);
}

sub kline_user($$$) {
	my ($user, $time, $reason) = @_;
	my $agent = get_user_agent($user);
	my ($ident, $host) = nickserv::get_host($user);

	ircd::kline($agent, '*', $host, $time, $reason);
}

# This is mostly for logging, be careful using it for anything else
sub get_hostmask($) {
	my ($user) = @_;
	my ($ident, $host);
	my $src = get_user_nick($user);
	
	($ident, $host) = get_host($user);

	return "$src!$ident\@$host";
}


sub expire {
	my ($ne, $e, $ve) = ($services_conf{'nearexpire'}, $services_conf{'nickexpire'}, $services_conf{'vacationexpire'});
	
	$get_expired->execute(time() - (86400 * $e), time() - (86400 * $ve));
	while(my ($nick, $email, $ident, $vhost) = $get_expired->fetchrow_array) {
		dropgroup($nick);
		log::wlog($nsnick, log::INFO(), "$nick has expired.  Email: $email  Vhost: $ident\@$vhost");
		chanserv::drop_nick_chans($nick);
	}

	my $time = time();

	$get_near_expired->execute($time - (86400 * ($e - $ne)), $time - (86400 * ($ve - $ne)));
	while(my ($nick, $email) = $get_near_expired->fetchrow_array) {
		my $aliases = get_aliases($nick);
		
		my $message = "We would like to remind you that your registered nick, $nick, will expire\nin $ne days unless you sign on and identify.";
		if(@$aliases > 1) {
			$message .= "\n\nThe following nicks are linked in this group:\n  " . join("\n  ", @$aliases);
		}
		
		email::send($email, "$nsnick Expiration Notice", $message);
		
		$set_near_expired->execute($time, 86400 * ($e - ($ne-1)), $time - 86400 * ($e - $ne), $nick);
		log::wlog($nsnick, log::INFO(), "$nick will expire in $ne days. ($email)");
	}
}

sub do_svssilence($$) {
	my ($user, $rootnick) = @_;
	my $target = get_user_nick($user);
	
	$get_silences->execute($rootnick);
	my $count = $get_silences->rows;
	unless ($get_silences->rows) {
		$get_silences->finish;
		return;
	}
	my @silences;
	for(my $i = 1; $i <= $count; $i++) {
		my ($mask, $time, $expiry) = $get_silences->fetchrow_array;
		push @silences, "+$mask";
	}
	$get_silences->finish;
	ircd::svssilence($nsnick, $target, @silences);
	return;
}

sub do_svswatch($$) {
	my ($user, $rootnick) = @_;
	my $target = get_user_nick($user);
	
	$get_watches->execute($rootnick);
	my $count = $get_watches->rows;
	unless ($get_watches->rows) {
		$get_watches->finish;
		return;
	}
	my @watches;
	for(my $i = 1; $i <= $count; $i++) {
		my ($mask, $time, $expiry) = $get_watches->fetchrow_array;
		push @watches, "+$mask";
	}
	$get_watches->finish;
	ircd::svswatch($nsnick, $target, @watches);
	return;
}

### DATABASE UTILITY FUNCTIONS ###

sub get_lock($) {
	my ($nick) = @_;
	
	$nick = lc $nick;

	$user_table++;

	if($cur_lock) {
		if($cur_lock ne $nick) {
			really_release_lock($nick);
			$user_table--;
			die("Tried to get two locks at the same time");
		}
		$cnt_lock++;
	} else {
		$cur_lock = $nick;
		$get_lock->execute($services_conf{'mysql-db'}.".user.$nick");
		$get_lock->finish;
	}
}

sub release_lock($) {
	my ($nick) = @_;
	
	$nick = lc $nick;

	$user_table--;

	if($cur_lock and $cur_lock ne $nick) {
		really_release_lock($cur_lock);
		
		die("Tried to release the wrong lock");
	}
	
	if($cnt_lock) {
		$cnt_lock--;
	} else {
		really_release_lock($nick);
	}
}

sub really_release_lock($) {
	my ($nick) = @_;

	$cnt_lock = 0;
	$release_lock->execute($services_conf{'mysql-db'}.".user.$nick");
	$release_lock->finish;
	undef $cur_lock;
}

sub set_user_flag($$;$) {
	my ($user, $flag, $sign) = @_;
	my $uid = get_user_id($user);
	$sign = 1 unless defined($sign);

	if($sign) {
		$set_flag->execute($flag, $flag, $uid);
	} else {
		$unset_flag->execute($flag, $flag, $uid);
	}
}

sub chk_user_flag($$;$) {
	my ($user, $flag, $sign) = @_;
	my $uid = get_user_id($user);
	$sign = 1 unless defined($sign);

	$get_user_flags->execute($uid);
	my ($flags) = $get_user_flags->fetchrow_array;

	return ($sign ? ($flags & $flag) : !($flags & $flag));
}

sub get_user_agent($) {
	my ($user) = @_;

	eval { $user->{AGENT} };
	if($@) {
		die("invalid get_user_agent call");
	}
	
	if(exists($user->{AGENT})) { 
		return $user->{AGENT}
	}
	else {
		return undef;
	}
}

sub get_host($) {
	my ($user) = @_;
	my $id = get_user_id($user);
	
	$get_host->execute($id);
	my ($ident, $host) = $get_host->fetchrow_array;

	return ($ident, $host);
}

sub get_vhost($) {
	my ($user) = @_;
	my $id = get_user_id($user);
	
	$get_vhost->execute($id);
	my ($ident, $vhost) = $get_vhost->fetchrow_array;

	return ($ident, $vhost);
}

sub get_user_info($) {
	my ($user) = @_;

	my $uid = get_user_id($user);
	return undef() unless $uid;

	$get_user_info->execute($uid);
	my ($ident, $host, $vhost, $gecos, $server) = $get_user_info->fetchrow_array();
	$get_user_info->finish;

	return ($ident, $host, $vhost, $gecos, $server);
}

sub cleanup() {
	$cleanup->execute();
}

sub cleanup_users() {
	add_timer('', OLD_USER_AGE(), __PACKAGE__, 'nickserv::cleanup_users');
	my $time = (time() - OLD_USER_AGE()*2);
	$cleanup_users->execute($time);
	$cleanup_nickid->execute($time);
}

sub fix_vhosts() {
	return; # XXX
	add_timer('fix_vhosts', 5, __PACKAGE__, 'nickserv::fix_vhosts');
	$get_hostless_nicks->execute();
	while (my ($nick) = $get_hostless_nicks->fetchrow_array) {
		ircd::notice($nsnick, $main_conf{'diag'}, "HOSTLESS NICK $nick");
		ircd::userhost($nick);
		ircd::userip($nick);
	}
	$get_hostless_nicks->finish();
}

### IRC EVENTS ###

sub killhandle($$$$) {
	my ($src, $dst, $path, $reason) = @_;
	
	unless(is_agent($dst)) {
		nick_delete($dst, "Killed ($src ($reason))");
	}
}

1;
